# -*- coding: utf-8 -*-
import numpy as np
import obspy
from obspy.io.sac import SACTrace
from obspy.signal.util import next_pow_2
from math import pi
from numpy.fft import fft, ifft, ifftshift
# from scipy.linalg import solve_toeplitz
import matplotlib.pyplot as plt


def gaussFilter(dt, nft, f0):
    df = 1.0 / (nft * dt)
    nft21 = 0.5 * nft + 1
    f = df * np.arange(0, nft21)
    w = 2 * pi * f

    gauss = np.zeros([nft, 1])
    gauss1 = np.exp(-0.25 * (w / f0) ** 2) / dt
    gauss1.shape = (len(gauss1), 1)
    gauss[0:int(nft21)] = gauss1
    gauss[int(nft21):] = np.flipud(gauss[1:int(nft21) - 1])
    gauss = gauss[:, 0]

    return gauss


def gfilter(x, nfft, gauss, dt):
    Xf = fft(x, nfft)
    Xf = Xf * gauss * dt
    xnew = ifft(Xf, nfft).real
    return xnew


def correl(R, W, nfft):
    x = ifft(fft(R, nfft) * np.conj(fft(W, nfft)), nfft)
    x = x.real
    return x


def phaseshift(x, nfft, dt, tshift):
    Xf = fft(x, nfft)
    shift_i = int(tshift / dt)
    p = 2 * pi * np.arange(1, nfft + 1) * shift_i / nfft
    Xf = Xf * np.vectorize(complex)(np.cos(p), -np.sin(p))
    x = ifft(Xf, nfft) / np.cos(2 * pi * shift_i / nfft)
    x = x.real
    return x


def deconit(uin, win, dt, nt=None, tshift=10, f0=2.0, itmax=400, minderr=0.001, phase='P'):
    """
    Created on Wed Sep 10 14:21:38 2014

    In:
    uin = numerator (radial for PdS)
    win = denominator (vertical component for PdS)
    dt = sample interval (s)
    nt = number of samples
    tshift = Time until beginning of receiver function (s)
    f0 = width of gaussian filter
    itmax = max # iterations
    minderr = Min change in error required for stopping iterations

    Out:
    RFI = receiver function
    rms = Root mean square error for predicting numerator after each iteration

    @author: Mijian Xu @ NJU
    """
    # print('Iterative Decon (Ligorria & Ammon):\n')
    if len(uin) != len(win):
        raise ValueError('The two input trace must be in same length')
    elif nt is None:
        nt = len(uin)
    else:
        pass

    rms = np.zeros(itmax)
    nfft = next_pow_2(nt)
    p0 = np.zeros(nfft)

    u0 = np.zeros(nfft)
    w0 = np.zeros(nfft)

    u0[0:nt] = uin
    w0[0:nt] = win

    gaussF = gaussFilter(dt, nfft, f0)
    # gaussF = _gauss_filter(dt, nfft, f0)

    u_flt = gfilter(u0, nfft, gaussF, dt)
    w_flt = gfilter(w0, nfft, gaussF, dt)

    wf = fft(w0, nfft)
    r_flt = u_flt

    powerU = np.sum(u_flt ** 2)

    it = 0
    sumsq_i = 1
    d_error = 100 * powerU + minderr
    maxlag = 0.5 * nfft
    # print('\tMax Spike Display is ' + str((maxlag) * dt))

    while np.abs(d_error) > minderr and it < itmax:
        rw = correl(r_flt, w_flt, nfft)
        rw = rw / np.sum(w_flt ** 2)

        if phase == 'P':
            i1 = np.argmax(np.abs(rw[0:int(maxlag) - 1]))
        else:
            i1 = np.argmax(np.abs(rw))
        amp = rw[i1] / dt

        p0[i1] = p0[i1] + amp
        p_flt = gfilter(p0, nfft, gaussF, dt)
        p_flt = gfilter(p_flt, nfft, wf, dt)

        r_flt = u_flt - p_flt
        sumsq = np.sum(r_flt ** 2) / powerU
        rms[it] = sumsq
        d_error = 100 * (sumsq_i - sumsq)

        sumsq_i = sumsq

        it = it + 1

    p_flt = gfilter(p0, nfft, gaussF, dt)
    p_flt = phaseshift(p_flt, nfft, dt, tshift)
    RFI = p_flt[0:nt]
    rms = rms[0:it - 1]

    return RFI, rms, it 
    

def deconwater(uin, win, dt, tshift=10., wlevel=0.05, f0=2.0, normalize=False, phase='P'):
    """
    Frequency-domain deconvolution using waterlevel method.

    :param uin: R or Q component for the response function
    :type uin: np.ndarray
    :param win: Z or L component for the source function
    :type win: np.ndarray
    :param dt: sample interval in second 
    :type dt: float
    :param tshift: Time shift before P arrival, defaults to 10.
    :type tshift: float, optional
    :param wlevel: Waterlevel to stabilize the deconvolution, defaults to 0.05
    :type wlevel: float, optional
    :param f0: Gauss factor, defaults to 2.0
    :type f0: float, optional
    :param normalize: If normalize the amplitude of the RF, defaults to False
    :type normalize: bool, optional

    :return: (rf, rms) RF and final rms.
    :rtype: (np.ndarray, float)
    """
    if uin.size != win.size:
        raise ValueError('The length of the \'uin\' must be same as the \'win\'')
    nt = uin.size
    nft = next_pow_2(nt)
    nfpts = nft / 2 + 1     # number of freq samples
    fny = 1. / (2.* dt);     # nyquist
    delf = fny / (0.5 * nft)
    freq = delf * np.arange(nfpts)
    w = 2 * pi * freq

    # containers
    # rff = np.zeros(nft); # Rfn in freq domain
    upf = np.zeros(nft); # predicted numer in freq domain

    # Convert seismograms to freq domain
    uf = fft(uin, nft)
    wf = fft(win, nft)

    # denominator
    df = wf * wf.conjugate()
    dmax = max(df.real);   

    # add water level correction
    phi1 = wlevel * dmax # water level
    # nwl = length( find(df<phi1) ) # number corrected
    df[np.where(df.real < phi1)[0]] = phi1
    gaussF = gaussFilter(dt, nft, f0)
    nf = gaussF * uf * wf.conjugate()

    # compute RF
    rff = nf / df
    
    # compute predicted numerator
    upf = rff * wf

    # add phase shift to RF
    w = np.append(w, -np.flipud(w[1:-1]))
    rff = rff * np.exp(-1j * w * tshift)

    # back to time domain
    rft = ifft(rff , nft)
    rft = rft[0:nt].real

    # compute the fit
    uf = gaussF * uf # compare to filtered numerator
    ut = ifft(uf, nft).real
    upt = ifft(upf, nft).real

    powerU = np.sum(ut[0:nt] ** 2)
    rms = np.sum((upt[0:nt] - ut[0:nt]) ** 2 )/powerU

    if normalize:
        gnorm = np.sum(gaussF) * delf * dt
        rft = rft.real / gnorm

    return rft, rms


def deconvolute(uin, win, dt, method='iter', **kwargs):
    if method.lower() == 'iter':
        return deconit(uin, win, dt, **kwargs)
    elif method.lower() == 'water':
        return deconwater(uin, win, dt, **kwargs)
    else:
        raise ValueError('method must be \'iter\' or \'water\'')


class RFTrace(obspy.Trace):
    def __init__(self, data=..., header=None):
        super().__init__(data=data, header=header)

    @classmethod
    def deconvolute(cls, utr, wtr, method='iter', **kwargs):
        header = utr.stats.__getstate__()
        for key, value in kwargs.items():
            header[key] = value
        if method.lower() == 'iter':
            rf, rms, it = deconit(utr.data, wtr.data, utr.stats.delta, **kwargs)
            header['rms'] = rms
            header['iter'] = it
        elif method.lower() == 'water':
            rf, rms = deconwater(utr.data, wtr.data, utr.stats.delta, **kwargs)
            header['rms'] = rms
            header['iter'] = np.nan
        else:
            raise ValueError('method must be \'iter\' or \'water\'')
        return cls(rf, header)
        



from seispy.ccp3d import CCP3D
from seispy.geo import extrema, latlon_from
from seispy.signal import smooth
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib.ticker import AutoMinorLocator, AutoLocator
from matplotlib.backend_bases import MouseButton
import warnings


def search_peak(tr, cpara, depmin, depmax):
    idx_all = extrema(tr)
    idx_ex = np.where(tr[idx_all] > 0)[0]
    peak = tr[idx_all[idx_ex]]
    peak_dep = idx_all[idx_ex] * cpara.stack_val + cpara.stack_range[0]
    idx_moho = np.where((peak_dep>depmin) & (peak_dep<depmax))[0]
    try:
        idx_moho_max = idx_moho[np.nanargmax(peak[idx_moho])]
        dep_moho = peak_dep[idx_moho_max]
    except:
        dep_moho = np.nan
    return dep_moho

class GoodDepth():
    def __init__(self, stack_data_path, logger, width=7.2, height=11, dpi=100, smooth=10) -> None:
        self.stack_data_path = stack_data_path
        self.logger = logger
        self.ccp_data = CCP3D.read_stack_data(stack_data_path)
        self.good_depth = pd.DataFrame(columns=['depth', 'amp', 'count', 'ci_low', 'ci_high'])
        self.bin_idx = 0
        self.smooth = smooth
        self.smooth_val = int(self.smooth/self.ccp_data.cpara.stack_val)
        self.create_fig(width=width, height=height, dpi=dpi)
        
    def create_fig(self, width=7.2, height=11, dpi=100):
        self.fig = plt.figure(figsize=(width, height), dpi=dpi, constrained_layout=True)
        self.gs = gridspec.GridSpec(10, 8, figure=self.fig)
        self.ax_ew = self.fig.add_subplot(self.gs[0:2, 0:8])
        self.ax_ns = self.fig.add_subplot(self.gs[2:, 0:2])
        self.ax_stack = self.fig.add_subplot(self.gs[2:, 2:6])
        self.ax_count = self.fig.add_subplot(self.gs[2:, 6:8])

    def get_dep(self, depmin, depmax):
        self.depmin = depmin
        self.depmax = depmax
        if depmin > self.ccp_data.cpara.stack_range[-1] or \
           depmax < self.ccp_data.cpara.stack_range[0] or \
           depmax < depmin:
           raise ValueError('Depth range is out of stacking range.')
        for i, bin_stack in enumerate(self.ccp_data.stack_data):
            tr = smooth(bin_stack['mu'], self.smooth_val, 'hanning')
            dep_moho = search_peak(tr, self.ccp_data.cpara, depmin, depmax)
            if np.isnan(dep_moho):
                stackamp = np.nan
                count = np.nan
                ci_low = np.nan
                ci_high = np.nan
            else:
                idx = int((dep_moho - self.ccp_data.cpara.stack_range[0])/self.ccp_data.cpara.stack_val)
                stackamp = bin_stack['mu'][idx]
                count = bin_stack['count'][idx]
                ci_low = bin_stack['ci'][idx, 0]
                ci_high = bin_stack['ci'][idx, 1]
            this_df = pd.DataFrame([[dep_moho, stackamp, count, ci_low, ci_high]],
                                   columns=['depth', 'amp', 'count', 'ci_low', 'ci_high'])
            self.good_depth = pd.concat([self.good_depth, this_df], ignore_index=True)
    
    def _get_adjacent_lim(self):
        offset = self.ccp_data.cpara.center_bin[-1]*self.val+0.1
        lat, lon = self.ccp_data.bin_loca[self.bin_idx]
        _, lon_max = latlon_from(lat, lon, 90, offset)
        _, lon_min= latlon_from(lat, lon, 270, offset)
        lat_max, _ = latlon_from(lat, lon, 0, offset)
        lat_min, _ = latlon_from(lat, lon, 180, offset)
        self.ew_lon_lim = [lon_min, lon_max]
        self.ns_lat_lim = [lat_min, lat_max]

    def get_adjacent(self, idx, val=5):
        self.val = val
        self.adjacent_ns = np.linspace(idx[0]-val, idx[0]+val, 2*val+1, dtype=np.uint8)
        self.adjacent_ew = np.linspace(idx[1]-val, idx[1]+val, 2*val+1, dtype=np.uint8)
        self.ns_lat = np.zeros(self.adjacent_ns.size)
        self.ew_lon = np.zeros(self.adjacent_ew.size)
        self.stack_ew = np.zeros([2*val+1, self.ccp_data.cpara.stack_range.size])
        self.stack_ns = np.zeros([2*val+1, self.ccp_data.cpara.stack_range.size])
        for i, idx_lat in enumerate(self.adjacent_ns):
            try:
                bin_idx = self.ccp_data.bin_map[idx_lat, idx[1]]
                self.stack_ns[i] = self.ccp_data.stack_data[bin_idx]['mu']
                self.ns_lat[i] = self.ccp_data.bin_loca[bin_idx, 0]
            except:
                self.stack_ns[i] = np.nan
                self.ns_lat[i] = np.nan
        for i, idx_lon in enumerate(self.adjacent_ew):
            try:
                bin_idx = self.ccp_data.bin_map[idx[0], idx_lon]
                self.stack_ew[i] = self.ccp_data.stack_data[bin_idx]['mu']
                self.ew_lon[i] = self.ccp_data.bin_loca[bin_idx, 1]
            except:
                self.stack_ew[i] = np.nan
                self.ew_lon[i] = np.nan
        self._get_adjacent_lim()

    def plot_bin(self, **kwargs):
        idx = np.where(self.ccp_data.bin_map==self.bin_idx)
        idx = [idx[0][0], idx[1][0]]
        if self.depmin < 100:
            self.plot_min = 0
        else:
            self.plot_min = self.depmin-(self.depmax-self.depmin)*0.15
        self.plot_max = self.depmax+(self.depmax-self.depmin)*0.15
        # self.this_depth = self.good_depth.iloc[self.bin_idx]['depth']
        self.get_adjacent(idx)
        self.init_fig(**kwargs)
        self.logger.PickDepthlog.info('{}/{}: Depth = {} km'.format(
            self.bin_idx+1, self.ccp_data.bin_loca.shape[0],
            self.good_depth.iloc[self.bin_idx]['depth']
        ))

    def init_fig(self):
        self.plot_ew(self.ax_ew)
        self.plot_ns(self.ax_ns)
        self.get_prot_moho()
        self.plot_stack(self.ax_stack, self.ax_count)
    
    def plot_ew(self, ax):
        ax.cla()
        ax.grid(color='gray', linestyle='--', linewidth=0.4, axis='y')
        for i, pos_lon in enumerate(self.ew_lon):
            # pos_lon = self.bin_mat[self.bin_loc_idx[0], idx_lon, 1]
            amp = self.stack_ew[i]*self.ccp_data.cpara.center_bin[-1]*self.val*0.5 + pos_lon
            ax.plot(amp, self.ccp_data.cpara.stack_range, linewidth=0.2, color='black')
            if not np.isnan(amp).all():
                if i == self.val:
                    ax.fill_betweenx(self.ccp_data.cpara.stack_range, amp, pos_lon, where=amp > pos_lon, facecolor='k', alpha=0.7)
                else:
                    ax.fill_betweenx(self.ccp_data.cpara.stack_range, amp, pos_lon, where=amp > pos_lon, facecolor='r', alpha=0.7)
        ax.set_ylim(self.plot_min, self.plot_max)
        ax.set_ylabel('Depth (km)')
        ax.set_xlim(self.ew_lon_lim)
        ax.set_xlabel('Longitude ($^\circ$)')
        ax.set_title('Index: {}, Lat: {:.2f}$^\circ$, Lon: {:.2f}$^\circ$'.format(
            self.bin_idx+1, self.ccp_data.bin_loca[self.bin_idx][0], self.ccp_data.bin_loca[self.bin_idx][1]))
        ax.invert_yaxis()

    def plot_ns(self, ax):
        ax.cla()
        ax.grid(color='gray', linestyle='--', linewidth=0.4, axis='x')
        for i, pos_lat in enumerate(self.ns_lat):
            # pos_lat = self.bin_mat[idx_lat, self.bin_loc_idx[1], 0]
            amp = self.stack_ns[i] * self.ccp_data.cpara.center_bin[-1]*self.val*0.5 + pos_lat
            ax.plot(self.ccp_data.cpara.stack_range, amp, linewidth=0.2, color='black')
            if not np.isnan(amp).all():
                if i == self.val:
                    ax.fill_between(self.ccp_data.cpara.stack_range, amp, pos_lat, where=amp > pos_lat, facecolor='k', alpha=0.7)
                else:
                    ax.fill_between(self.ccp_data.cpara.stack_range, amp, pos_lat, where=amp > pos_lat, facecolor='red', alpha=0.7)
        ax.set_xlim(self.plot_min, self.plot_max)
        # ax.set_xticks(np.arange(0, 100, 20))
        ax.set_xlabel('Depth (km)')
        ax.set_ylim(self.ns_lat_lim)
        # ax.set_ylim(self.ns_lat[0]-0.1, self.ns_lat[-1]+0.1)
        ax.set_ylabel('Latitude ($^\circ$)')

    def get_prot_moho(self):
        # bin_stack = self.ccp.stack_data[i]
        self.mu = self.ccp_data.stack_data[self.bin_idx]['mu']
        self.ci = self.ccp_data.stack_data[self.bin_idx]['ci']
        if self.smooth is not None:
            self.mu = smooth(self.mu, self.smooth_val, 'hanning')
            self.ci[:, 0] = smooth(self.ci[:, 0], self.smooth_val, 'hanning')
            self.ci[:, 1] = smooth(self.ci[:, 1], self.smooth_val, 'hanning')
        if np.isnan(self.mu).all():
            self.prot_moho = [np.nan]
            self.maxpeak = [np.nan]
        else:
            idx_all = extrema(self.mu)
            self.ex_idx = idx_all[np.where(self.mu[idx_all] > 0)[0]]
            self.prot_moho = self.ccp_data.cpara.stack_range[self.ex_idx]
            #  self.prot_moho = self.prot_moho[np.where(self.mu[self.prot_moho] > 0)]
            # self.prot_moho.sort()
            self.maxpeak = np.nanmax(self.mu) + 0.1

    def plot_stack(self, ax, ax_c):
        warnings.simplefilter('ignore')
        ax.cla()
        ax_c.cla()
        maxamp = np.nanmax(np.abs(self.mu))*1.5
        ax.plot([0, 0], [0, 1000], color='k', linewidth=0.5)
        ax.plot(self.mu, self.ccp_data.cpara.stack_range)
        ax.plot(self.ci[:, 0], self.ccp_data.cpara.stack_range, lw=0.5, ls='--', color='black')
        ax.plot(self.ci[:, 1], self.ccp_data.cpara.stack_range, lw=0.5, ls='--', color='black')
        for extr in self.prot_moho:
            ax.plot([-maxamp, maxamp], [extr, extr], linewidth=2, color='tomato')
        self.pltdepth, = ax.plot([-maxamp, maxamp], [self.good_depth.iloc[self.bin_idx]['depth']]*2, linewidth=2, color='g')
        # ax.set_xlim([-self.maxpeak, self.maxpeak])
        ax.set_xlabel('Amplitude')
        ax.set_ylim([self.plot_min, self.plot_max])
        # ax.set_yticks(np.arange(self.ccp_data.cpara.stack_range[0], self.ccp_data.cpara.stack_range[-1], 10))
        ax.set_ylabel('Depth (km)')
        ax.yaxis.set_major_locator(AutoLocator())
        ax.yaxis.set_minor_locator(AutoMinorLocator())
        ax.invert_yaxis()

        ax_c.barh(self.ccp_data.cpara.stack_range, self.ccp_data.stack_data[self.bin_idx]['count'], height=0.8, facecolor='tan')
        ax_c.set_ylim([self.plot_min, self.plot_max])
        # ax_c.set_yticks(ax.get_yticks())
        ax.yaxis.set_minor_locator(AutoMinorLocator())
        ax_c.yaxis.set_minor_locator(AutoMinorLocator())
        ax_c.set_xlabel('Count')
        ax_c.invert_yaxis()

    def _check_nan_bin(self):
        while True:
            if np.isnan(self.good_depth.iloc[self.bin_idx]['depth']):
                self.bin_idx += 1
            else:
                break

    def _get_next_bin(self):
        self.bin_idx += 1
        while True:
            # mu = self.ccp_data.stack_data[self.bin_idx]['mu']
            if self.bin_idx >= self.ccp_data.bin_loca.shape[0]:
                self.bin_idx = self.ccp_data.bin_loca.shape[0]-1
                break
            if np.isnan(self.good_depth.iloc[self.bin_idx]['depth']):
                self.bin_idx += 1
            else:
                break
    
    def _get_previous_bin(self):
        self.bin_idx -= 1
        while True:
            # mu = self.ccp_data.stack_data[self.bin_idx]['mu']
            if self.bin_idx < 0:
                self.bin_idx = 0
                break
            if np.isnan(self.good_depth.iloc[self.bin_idx]['depth']):
                self.bin_idx -= 1
            else:
                break
    
    def page_up(self, **kwargs):
        self._get_previous_bin()
        self.plot_bin(**kwargs)
    
    def page_down(self, **kwargs):
        self._get_next_bin()
        self.plot_bin(**kwargs)

    def on_click(self, event):
        if event.inaxes == self.ax_stack:
            idx = np.argmin(np.abs(self.prot_moho - event.ydata))
            sub_value = np.min(np.abs(self.prot_moho - event.ydata))
            if event.button is MouseButton.LEFT:
                if sub_value < 10:
                    self.pltdepth.set_ydata([self.prot_moho[idx]]*2)
                    self.pltdepth.set_visible(True)
                    self.logger.PickDepthlog.info('Set the depth to {} km'.format(self.prot_moho[idx]))
                    self.good_depth.iloc[self.bin_idx]['depth'] = self.prot_moho[idx]
            elif event.button is MouseButton.RIGHT:
                self.pltdepth.set_visible(False)
                self.logger.PickDepthlog.info('Set the depth to NaN')
                self.good_depth.iloc[self.bin_idx]['depth'] = np.nan

    def write(self, fname):
        bin_df = pd.DataFrame(self.ccp_data.bin_loca, columns=['lat', 'lon'])
        full_pd = pd.concat([bin_df, self.good_depth['depth'],
                  self.good_depth['ci_low'], self.good_depth['ci_high'],
                  self.good_depth['count']], axis=1)
        full_pd.to_csv(fname, sep=' ', header=False,
                       index=False, na_rep='nan',
                       float_format='%.4f')


from seispy.ccppara import ccppara
from seispy.ccpprofile import CCPProfile
from seispy.rf2depth_makedata import makedata
from os.path import exists
from subprocess import Popen
import pytest
import os

def test_download():
    if exists('ex-ccp.tar.gz'):
        pytest.skip('Data are downloaded.')
    s = 'wget https://osf.io/hzq2x/download -O ex-ccp.tar.gz\n'
    s += 'tar -xzf ex-ccp.tar.gz\n'
    proc = Popen(s, shell=True)
    proc.communicate()


def test_sub01():
    para = ccppara('ex-ccp/ccp.cfg')
    para.rfpath = 'ex-ccp/RFresult'
    para.stalist = 'ex-ccp/sta.lst'
    para.stack_sta_list = ''
    makedata(para)


def test_sub02():
    ccp = CCPProfile('ex-ccp/ccp.cfg')
    # para.stalist = 'ex-ccp/sta.lst'
    ccp.cpara.width = 40
    ccp.initial_profile()
    ccp.stack()
    ccp.save_stack_data(format='dat')
    os.remove(ccp.cpara.stack_sta_list)
    os.remove(ccp.cpara.stackfile)

def test_sub03():
    ccp = CCPProfile('ex-ccp/ccp.cfg')
    ccp.cpara.adaptive = True
    ccp.cpara.stack_sta_list = 'ex-ccp/sta.lst'
    ccp.initial_profile()
    ccp.stack()
    ccp.save_stack_data(format='dat')
    os.remove(ccp.cpara.stackfile)

if __name__ == '__main__':
    # test_download()
    test_sub01()


import pandas as pd
from obspy import UTCDateTime, Catalog
from obspy.clients.fdsn import Client


def _cat2df(cat):
    cols = ['date', 'evla', 'evlo', 'evdp', 'mag', 'magtype']
    data = [[evt.origins[0].time, evt.origins[0].latitude, evt.origins[0].longitude, evt.origins[0].depth*0.001,
             evt.magnitudes[0].mag, evt.magnitudes[0].magnitude_type] for evt in cat if evt.origins[0].depth is not None]
    return pd.DataFrame(data, columns=cols)


class Query():
    def __init__(self, server='IRIS'):
        self.client = Client(server)

    def get_events(self, starttime=None,
                   endtime=UTCDateTime.now(), 
                   **kwargs):
        if endtime > UTCDateTime.now():
            endtime = UTCDateTime.now()
        events = Catalog()
        if endtime-starttime < 365 * 86400:
            events += self.client.get_events(starttime=starttime,
                                            endtime=endtime,
                                            orderby='time-asc', **kwargs)
        else:
            chunk_length = 365 * 86400
            while starttime <= endtime:
                events += self.client.get_events(starttime=starttime,
                                                endtime=starttime + chunk_length,
                                                orderby='time-asc', **kwargs)
                if starttime + chunk_length > endtime:
                    chunk = endtime - starttime
                    if chunk <= 1:
                        break
                starttime += chunk_length

        self.events = _cat2df(events)
        self.events_raw = events

    def get_stations(self, includerestricted=False, **kwargs):
        self.stations = self.client.get_stations(includerestricted=includerestricted, **kwargs)





import os
from seispy.rf import RF, read_catalog
from seispy.recalrf import ReRF
from seispy.hk import hksta
from seispy.hkpara import HKPara
from seispy.catalog import download_catalog
from subprocess import Popen
import pytest
from os.path import exists, join, basename
import glob
from obspy.io.sac import SACTrace
from obspy import UTCDateTime


def test_download():
    if exists('ex-prf.tar.gz'):
        pytest.skip('Data are downloaded.')
    s = 'wget https://osf.io/dxcfz/download -O ex-prf.tar.gz\n'
    s += 'tar -xzf ex-prf.tar.gz\n'
    proc = Popen(s, shell=True)
    proc.communicate()

def init_RF():
    rf = RF(cfg_file='ex-prf/rf.cfg')
    rf.para.phase = 'P'
    rf.para.datapath = 'ex-prf/Data.CB.NJ2'
    rf.para.rfpath = 'ex-prf/RFresult/CB.NJ2'
    rf.load_stainfo()
    rf.search_eq()
    rf.match_eq()
    rf.detrend()
    rf.filter()
    rf.cal_phase()
    rf.drop_eq_snr()
    rf.baz_correct()
    return rf

def gen_list(para):
    with open(os.path.join(para.rfpath, "CB.NJ2finallist.dat"), 'w+') as fid:
        files = sorted(glob.glob(join(para.rfpath, '*R.sac')))
        for fname in files:
            sac = SACTrace.read(fname)
            evname = basename(fname).split('_')[0]
            fid.write('%s %s %6.3f %6.3f %6.3f %6.3f %6.3f %8.7f %6.3f %6.3f\n' % (
                evname, 'P', sac.evla, sac.evlo, sac.evdp, sac.gcarc, sac.baz, sac.user0, sac.mag, sac.user1
            ))


def test_sub01():
    rf = init_RF()
    rf.para.decon_method = 'water'
    rf.para.criterion = 'crust'
    rf.para.rmsgate = None
    rf.rotate()
    rf.trim()
    rf.deconv()
    rf.saverf()


def test_sub02():
    rf = init_RF()
    rf.para.comp = 'lqt'
    rf.para.decon_method = 'water'
    rf.para.criterion = None
    rf.para.rmsgate = None
    rf.rotate()
    rf.trim()
    rf.deconv()
    rf.saverf()


def test_sub03():
    rf = init_RF()
    rf.para.decon_method = 'iter'
    rf.para.criterion = 'crust'
    rf.para.rmsgate = 0.2
    rf.rotate()
    rf.trim()
    rf.deconv()
    rf.saverf()
    gen_list(rf.para)


def test_sub04():
    rf = ReRF('ex-prf/RFresult/CB.NJ2/CB.NJ2finallist.dat',
              cfg_file='ex-prf/rf.cfg')
    rf.para.phase = 'P'
    rf.para.datapath = 'ex-prf/Data.CB.NJ2'
    rf.para.rfpath = 'ex-prf/RFresult/CB.NJ2_re'
    rf.para.decon_method = 'iter'
    rf.para.gauss = 2.5
    rf.para.criterion = None
    rf.para.rmsgate = None
    rf.load_stainfo()
    rf.match_eq()
    rf.detrend()
    rf.filter()
    rf.cal_phase()
    rf.rotate()
    rf.trim()
    rf.deconv()
    rf.saverf()
    rf.write_list()


def test_sub05():
    hkpara = HKPara()
    hkpara.rfpath = 'ex-prf/RFresult/CB.NJ2'
    hkpara.hkpath = './'
    hksta(hkpara, isplot=True, isdisplay=True)


def test_sub06():
    fname = 'evts.lst'
    b_time = UTCDateTime('20200101')
    e_time = UTCDateTime('20200201')
    download_catalog(fname, format='QUAKEML', starttime=b_time,
                     endtime=e_time, minmagnitude=5.5)
    eq_lst = read_catalog(fname, b_time, e_time, 0.,0.,)
    print(eq_lst)


if __name__ == '__main__':
    test_sub06()


import numpy as np
import argparse
from seispy.rfcorrect import RFStation
from scipy.interpolate import interp1d
from seispy.utils import read_rfdep
from seispy.rf import RF
from seispy.recalrf import ReRF


def rfharmo():
    parser = argparse.ArgumentParser('Harmonic decomposition for extracting anisotropic and isotropic features from the radial and transverse RFs')
    parser.add_argument('rfpath', type=str, help="Path to PRFs")
    parser.add_argument('-t', help="Time window from tb to te for triming RFs, NOTE: do not insert space before this argument, defaults to -2/10",
                        metavar='tb/te', default='-2/10')
    parser.add_argument('-s', help="Resample RFs with sampling interval of dt", metavar='dt', default=None, type=float)
    parser.add_argument('-o', help="Specify output path for saving constant component.", metavar='outpath', default=None, type=str)
    parser.add_argument('-e', help='Enlarge factor, defaults to 2', metavar='enf', default=2., type=float)
    parser.add_argument('-p', help='Figure output path, defaults to ./', metavar='figure_path', default='./', type=str)
    args = parser.parse_args()
    rfsta = RFStation(args.rfpath)
    if args.s is not None:
        rfsta.resample(args.s)
    twin = [float(v) for v in args.t.split('/')]
    rfsta.harmonic(twin[0], twin[1])
    if args.o is not None:
        rfsta.harmo.write_constant(args.o)
    rfsta.harmo.plot(outpath=args.p, enf=args.e)


def rfani():
    parser = argparse.ArgumentParser(description="Estimate crustal anisotropy with a Joint inversion method. See Liu and Niu (2012) in detail.")
    parser.add_argument('rfpath', type=str, help="Path to PRFs")
    parser.add_argument('-t', help="Time window for searching Pms from tb to te", metavar='tb/te', required=True)
    parser.add_argument('-c', help="List file in text format for saving results, defaults to ./rfani.dat",
                        default="rfani.dat", metavar="list_file_name")
    parser.add_argument('-l', help="Half length of time window cut around Pms phase, defaults to 3s",
                        default=3, metavar="half_time_length", type=float)
    parser.add_argument('-r', help='Ray-parameter for moveout correction, defaults to 0.06 s/km',
                        default=0.06, metavar="rayp")
    parser.add_argument('-m', help='velocity model for moveout correction. \'iasp91\', \'prem\''
                        'and \'ak135\' is valid for internal model. Specify path to velocity model for the customized model.' 
                        'The format is the same as in Taup, but the depth should be monotonically increasing, defaults to \'iasp91\'',
                        default='iasp91', metavar="velocity_model")
    parser.add_argument('-o', dest='outpath', help="Directory to the image, defaults to current directory.", default='./')
    parser.add_argument('-p', help="If plot RFs stacked by back-azimuth, defaults to \'False\'",
                        dest="isplot", action='store_true', default=False)
    parser.add_argument('-w', help="Weights of 3 anisotropic methods (order by R cosine energy, R cross-correlation and T energy), defaults to 0.4/0.4/0.2",
                        dest='weight', default='0.4/0.4/0.2', metavar='w1/w2/w3')
    arg = parser.parse_args()
    weights = np.array(arg.weight.split('/')).astype(float)
    timewin = np.array(arg.t.split('/')).astype(float)
    rfsta = RFStation(arg.rfpath)
    bf, bt = rfsta.jointani(timewin[0], timewin[1], tlen=arg.l, weight=weights)
    if bf.size == 1:
        bf = np.append(bf, np.nan)
        bt = np.append(bt, np.nan)
    with open(arg.c, 'a+') as fid:
        fid.write('{}\t{:.3f}\t{:.3f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\n'.format(
                  rfsta.staname, rfsta.stla, rfsta.stlo, bf[0], bt[0], bf[1], bt[1]))
    if arg.isplot:
        rfsta.ani.plot_stack_baz(outpath=arg.outpath)
    rfsta.ani.plot_polar(outpath=arg.outpath)


def ccp3d():
    from seispy.ccp3d import CCP3D
    parser = argparse.ArgumentParser(description="3-D CCP stacking with spaced grid bins.")
    parser.add_argument('cfg_file', type=str, help='Path to CCP configure file')
    parser.add_argument('-s', help='Range for searching depth of D410 and D660, The results would be saved to \'peakfile\' in cfg_file',
                        metavar='d410min/d410max/d660min/d660max', default=None)
    arg = parser.parse_args()
    ccp = CCP3D(arg.cfg_file)
    ccp.initial_grid()
    ccp.stack()
    ccp.save_stack_data(ccp.cpara.stackfile)
    if arg.s:
        search_range = np.array(arg.s.split('/')).astype(float)
        ccp.search_good_410_660(*search_range)
        ccp.save_good_410_660(ccp.cpara.peakfile)


def ccp_profile():
    from seispy.ccpprofile import CCPProfile
    parser = argparse.ArgumentParser(description="Stack PRFS along a profile")
    parser.add_argument('cfg_file', type=str, help='Path to CCP configure file')
    parser.add_argument('-t', help='Output as a text file', dest='isdat', action='store_true')
    arg = parser.parse_args()
    if arg.isdat:
        typ = 'dat'
    else:
        typ = 'npz'
    ccp = CCPProfile(arg.cfg_file)
    ccp.initial_profile()
    ccp.stack()
    ccp.save_stack_data(format=typ)


def get_pierce_points():
    parser = argparse.ArgumentParser(description="Get pierce points with assumed depth")
    parser.add_argument('rfdepth_path', help="path to rfdepth file")
    parser.add_argument('-d', help="The depth in km", type=float, metavar='depth')
    parser.add_argument('-o', help="filename of output file, defaults to ./pierce_points.dat",
                        default='./pierce_points.dat', metavar='filename')
    arg = parser.parse_args()
    rfdep = read_rfdep(arg.rfdepth_path)
    if arg.d > rfdep[0]['depthrange'][-1]:
        parser.error('The depth exceed max depth in {}'.format(arg.rfdepth_path))
    with open(arg.o, 'w') as f:
        for sta in rfdep:
            for i in range(sta['piercelat'].shape[0]):
                la = interp1d(sta['depthrange'], sta['piercelat'][i])(arg.d)
                lo = interp1d(sta['depthrange'], sta['piercelon'][i])(arg.d)
                f.write('{:.4f} {:.4f}\n'.format(lo, la))
    

def common_parser():
    parser = argparse.ArgumentParser(description="Calculating RFs for single station")
    parser.add_argument('cfg_file', type=str, help='Path to RF configure file')
    parser.add_argument('-l', help="use local catalog, defaults to false", dest='islocal', action='store_true')
    parser.add_argument('-r', help='Reverse components: N, E or NE', dest='comp',
                        metavar='N|E|NE', default=None, type=str)
    parser.add_argument('-s', help='Switch the East and North components', dest='isswitch', action='store_true')
    parser.add_argument('-b', help='Correct back-azimuth. \nIf "baz" is specified, the corr_baz = raw_baz + baz. \n'
                                   'If there is no argument, the back-azimuth will be corrected with minimal '
                                   'energy of T component. The searching range is raw_baz +/- 90',
                                   dest='baz', nargs='?', const=0, type=float)
    parser.add_argument('-w', help='Write project to localfile', action='store_true')
    return parser


def parse_common_args(args):
    if args.comp is not None:
        args.comp = args.comp.upper()
        if args.comp == 'NE' or args.comp == 'EN':
            reverseE = True
            reverseN = True
        elif args.comp == 'E':
            reverseE = True
            reverseN = False
        elif args.comp == 'N':
            reverseE = False
            reverseN = True
        else:
            raise ValueError('component name must be in EN, E or N')
    else:
        reverseN = False
        reverseE = False
    return reverseE, reverseN


def prf():
    parser = common_parser()
    parser.add_argument('-f', help='Specify finallist for re-calculating RFs and -l is invalid in this pattern',
                        metavar='finallist', default=None)
    arg = parser.parse_args()
    if arg.f is not None:
        arg.islocal = False
        pjt = ReRF(arg.f, cfg_file=arg.cfg_file)
    else:
        pjt = RF(cfg_file=arg.cfg_file)
    pjt.para.switchEN = arg.isswitch
    pjt.para.reverseE ,pjt.para.reverseN= parse_common_args(arg)
    pjt.load_stainfo()
    if arg.f is None:
        pjt.search_eq(local=arg.islocal)
    pjt.match_eq()
    pjt.channel_correct()
    pjt.detrend()
    pjt.filter()
    pjt.cal_phase()
    if arg.f is None:
        pjt.drop_eq_snr()
    if arg.baz is not None and arg.baz != 0:
        pjt.baz_correct(correct_angle=arg.baz)
    elif arg.baz is not None and arg.baz == 0:
        pjt.baz_correct()
    else:
        pass
    if arg.w:
        pjt.savepjt()
    pjt.rotate()
    pjt.trim()
    pjt.deconv()
    pjt.saverf()
    if arg.f is not None:
        pjt.write_list()


def srf():
    parser = common_parser()
    parser.add_argument('-p', help='Wether or not manually pick arrival time and waveforms arround S phase with a GUI.',
                        action='store_true')
    parser.add_argument('-i', help='Wether grid search incidence angle',
                        action='store_true')
    arg = parser.parse_args()
    pjt = RF(cfg_file=arg.cfg_file)

    pjt.para.switchEN = arg.isswitch
    pjt.para.reverseE ,pjt.para.reverseN= parse_common_args(arg)
    pjt.load_stainfo()
    pjt.search_eq(local=arg.islocal)
    pjt.match_eq()
    pjt.channel_correct()
    pjt.detrend()
    pjt.filter()
    pjt.cal_phase()
    pjt.drop_eq_snr()
    if arg.baz is not None and arg.baz != 0:
        pjt.baz_correct(correct_angle=arg.baz)
    elif arg.baz is not None and arg.baz == 0:
        pjt.baz_correct()
    else:
        pass
    pjt.rotate(search_inc=arg.i)
    if arg.p:
        pjt.pick()
    if arg.w:
        pjt.savepjt()
    pjt.trim()
    pjt.deconv()
    pjt.saverf()


def plot_rt():
    from seispy.plotRT import plotrt
    parser = argparse.ArgumentParser(description="Plot R(Q)&T components for P receiver functions (PRFs)")
    parser.add_argument('rfpath', help='Path to PRFs with a \'finallist.dat\' in it', type=str, default=None)
    parser.add_argument('-e', help='Enlargement factor, defaults to 3', dest='enf', type=float, default=3, metavar='enf')
    parser.add_argument('-o', help='Output path without file name, defaults to current path', dest='output', default='./', type=str, metavar='outpath')
    parser.add_argument('-t', help='Specify figure format. f = \'.pdf\', g = \'.png\', defaults to \'g\'',
                        dest='format', default='g', type=str, metavar='f|g')
    parser.add_argument('-x', help='The max time scale in sec, defaults to 30s', default=30, type=float, metavar='max_time')
    arg = parser.parse_args()
    if arg.format not in ('f', 'g'):
        raise ValueError('Error: The format must be in \'f\' and \'g\'')
    rfsta = RFStation(arg.rfpath)
    plotrt(rfsta, enf=arg.enf, out_path=arg.output, outformat=arg.format, xlim=[-2, arg.x])


def plot_r():
    from seispy.plotR import plotr
    parser = argparse.ArgumentParser(description="Plot R&T receiver functions")
    parser.add_argument('rfpath', help='Path to PRFs with a \'finallist.dat\' in it', type=str)
    parser.add_argument('-e', help='Enlargement factor, defaults to 6', dest='enf', type=float, default=6, metavar='enf')
    parser.add_argument('-o', help='Output path without file name, defaults to current path', dest='output', default='./', type=str, metavar='outpath')
    parser.add_argument('-t', help='Specify figure format. f = \'.pdf\', g = \'.png\', defaults to \'g\'',
                    dest='format', default='g', type=str, metavar='f|g')
    parser.add_argument('-x', help='The max time scale in sec, defaults to 85s', default=85, type=float, metavar='max_time')

    arg = parser.parse_args()
    if arg.format not in ('f', 'g'):
        parser.error('Error: The format must be in \'f\' and \'g\'')
    elif arg.format == 'g':
        fmt = 'png'
    elif arg.format == 'f':
        fmt = 'pdf'
    rfsta = RFStation(arg.rfpath)
    plotr(rfsta, arg.output, enf=arg.enf, xlim=[-2, arg.x], format=fmt)




from os.path import expanduser, join, exists, dirname
import os
import obspy
from obspy.io.sac import SACTrace
import glob
import configparser
from seispy.io import Query
import re


def _load_station_info(pathname, ref_comp, suffix):
    ex_sac = glob.glob(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
    if len(ex_sac) == 0:
        raise FileNotFoundError('no such SAC file in {0}'.format(pathname))
    for sac in ex_sac:
        try:
            ex_tr = SACTrace.read(sac, headonly=True)
            break
        except Exception:
            continue
    if (ex_tr.stla is None or ex_tr.stlo is None):
        raise ValueError('The stlo and stla are not in the SACHeader')
    return ex_tr.knetwk, ex_tr.kstnm, ex_tr.stla, ex_tr.stlo, ex_tr.stel


class StaInfo():
    def __init__(self):
        self.network = ''
        self.station = ''
        self.stla = 0.
        self.stlo = 0.
        self.stel = 0.
        self.channel = '*'
        self.location = '*'

    def get_stainfo(self):
        return self.__dict__

    def load_stainfo(self, pathname, ref_comp, suffix):
        (self.network, self.station, self.stla, self.stlo, self.stel) = _load_station_info(pathname, ref_comp, suffix)

    def get_station_from_ws(self, server='IRIS', **kwargs):
        """Get station information from web-service with given network and station or other optional condition.

        Parameters
        ----------
        server : str, optional
            Sever name of FDSN web-service, by default 'IRIS'
        """
        self.query = Query(server)
        self.query.get_stations(network=self.network, station=self.station,
                                channel=self.channel, location=self.location, 
                                level='channel', **kwargs)
        if len(self.query.stations) != 1 or len(self.query.stations[0]) != 1:
            raise ValueError('More than one station are fetched. Please set up stricter condition in query')
        self.stla = self.query.stations[0][0].latitude
        self.stlo = self.query.stations[0][0].longitude
        self.stel = self.query.stations[0][0].elevation


class RFPara(object):
    def __init__(self):
        self.datapath = expanduser('~')
        self.rfpath = expanduser('~')
        self.catalogpath = join(dirname(__file__), 'data', 'EventCMT.dat')
        self.pjtpath = 'rfpjt.pkl'
        self.cata_server = 'IRIS'
        self.data_server = 'IRIS'
        self.velmod = 'iasp91'
        self.offset = None
        self.tolerance = 210
        self.dateformat = '%Y.%j.%H.%M.%S'
        self.date_begin = obspy.UTCDateTime('19760101')
        self.date_end = obspy.UTCDateTime.now()
        self.magmin = 5.5
        self.magmax = 10
        self.dismin = 30
        self.dismax = 90
        self.ref_comp = 'BHZ'
        self.suffix = 'SAC'
        self.noisegate = 5
        self.noiselen = 50
        self.phase = 'P'
        self.gauss = 2.0
        self.target_dt = 0.01
        self.time_before = 10
        self.time_after = 120
        self.freqmin = 0.05
        self.freqmax = 1
        self.comp = 'RTZ'
        self.decon_method = 'iter'
        self.wlevel = 0.05
        self.itmax = 400
        self.minderr = 0.001
        self.criterion = None
        self.only_r = False
        self.rmsgate = None
        self.switchEN=False
        self.reverseE=False
        self.reverseN=False
        self.use_remote_data=False
        self.stainfo = StaInfo()

    def get_para(self):
        return self.__dict__

    def __str__(self):
        head = ['{}: {}'.format(k, v) for k, v in self.__dict__.items()]
        return '\n'.join(head)

    def _check_date_range(self):
        if self.date_begin > self.stainfo.query.stations[0][0].end_date or \
            self.date_end < self.stainfo.query.stations[0][0].start_date:
            raise ValueError('The date range set does not overlap with'
                             'the date range of the fetched data')
        if self.date_begin < self.stainfo.query.stations[0][0].start_date:
            self.date_begin = self.stainfo.query.stations[0][0].start_date
        if self.date_end > self.stainfo.query.stations[0][0].end_date:
            self.date_end = self.stainfo.query.stations[0][0].end_date

    @property
    def datapath(self):
        return self._datapath

    @datapath.setter
    def datapath(self, value):
        if not isinstance(value, str):
            raise TypeError('datapath should be \'str\' type not \'{0}\''.format(type(value)))
        else:
            self._datapath = value

    # Temporary function. It will be removed after 2~3 versions
    @property
    def server(self):
        return self.cata_server
    
    @server.setter
    def server(self, value):
        self.cata_server = value

    @property
    def rfpath(self):
        return self._rfpath

    @rfpath.setter
    def rfpath(self, value):
        if not isinstance(value, str):
            raise TypeError('rfpath should be \'str\' type not \'{0}\''.format(type(value)))
        elif not exists(value):
            try:
                os.makedirs(value)
            except Exception as e:
                Warning('Cannot create rfpath of {0}\n with error: {1}'.format(value, e))
            finally:
                self._rfpath = value
        else:
            self._rfpath = value

    @property
    def catalogpath(self):
        return self._catalogpath

    @catalogpath.setter
    def catalogpath(self, value):
        if not isinstance(value, str):
            raise TypeError('catalogpath should be \'str\' type not \'{0}\''.format(type(value)))
        self._catalogpath = value

    @property
    def criterion(self):
        return self._criterion

    @criterion.setter
    def criterion(self, value):
        if self.phase == 'S':
            if value == '' or value is None:
                self._criterion = None
            elif value.lower() == 'lab':
                self._criterion = 'lab'
            else:
                raise ValueError('criterion should be \'lab\'')
        else:
            if ''.join(sorted(self.comp.upper())) == 'LQT':
                self._criterion = None
            elif value == '' or value is None:
                self._criterion = None
            elif value.lower() not in ['crust', 'mtz']:
                raise ValueError('criterion should be string in \'crust\' or  \'mtz\'')
            else:
                self._criterion = value.lower()

    @property
    def decon_method(self):
        return self._decon_method
    
    @decon_method.setter
    def decon_method(self, value):
        if value not in ['iter', 'water']:
            raise ValueError('decon_method must be in \'iter\' or \'water\'')
        else:
            self._decon_method = value

    @classmethod
    def read_para(cls, cfg_file):
        cf = configparser.RawConfigParser(allow_no_value=True)
        pa = cls()
        try:
            cf.read(cfg_file)
        except Exception:
            raise FileNotFoundError('Cannot open configure file %s' % cfg_file)
        sections = cf.sections()
        for key, value in cf.items('path'):
            if value == '':
                continue
            elif key == 'datapath':
                pa.datapath = value
            elif key == 'rfpath':
                pa.rfpath = value
            elif key == 'catalogpath':
                pa.catalogpath = value
            else:
                pa.__dict__[key] = value
        sections.remove('path')
        if 'fetch' in sections:
            for key, value in cf.items('fetch'):
                pa.stainfo.__dict__[key] = value
            sections.remove('fetch')
        for sec in sections:
            for key, value in cf.items(sec):
                if key == 'date_begin':
                    pa.__dict__[key] = obspy.UTCDateTime(value)
                elif key == 'date_end':
                    pa.__dict__[key] = obspy.UTCDateTime(value)
                elif key == 'offset':
                    try:
                        pa.__dict__[key] = float(value)
                    except:
                        pa.__dict__[key] = None
                elif key == 'itmax':
                    pa.__dict__[key] = int(value)
                elif key == 'only_r':
                    pa.__dict__[key] = cf.getboolean(sec, 'only_r')
                elif key == 'criterion':
                    pa.criterion = value
                elif key == 'decon_method':
                    pa.decon_method = value
                elif key == 'gauss':
                    try:
                        pa.gauss = cf.getfloat(sec, 'gauss')
                    except:
                        pa.gauss = [float(v) for v in re.split(' |,', value)]
                elif key == 'rmsgate':
                    try:
                        pa.rmsgate = cf.getfloat(sec, 'rmsgate')
                    except:
                        pa.rmsgate = None
                else:
                    try:
                        exec('pa.{} = {}'.format(key, float(value)))
                        # pa.__dict__[key] = float(value)
                    except ValueError:
                        exec('pa.{} = value'.format(key))
                        # pa.__dict__[key] = value
        return pa


from seispy.rfcorrect import RFStation
import pytest
from os.path import exists
from subprocess import Popen


def test_download():
    if exists('ex-rfani.tar.gz'):
        pytest.skip('Data are downloaded.')
    s = 'wget https://osf.io/4hk6d/download -O ex-rfani.tar.gz\n'
    s += 'tar -xzf ex-rfani.tar.gz\n'
    proc = Popen(s, shell=True)
    proc.communicate()


def test_sub01():
    rfs = RFStation('ex-rfani/SC.LTA')
    rfs.jointani(3, 8, tlen=3.5, stack_baz_val=10, weight=[0.4, 0.4, 0.2])


def test_sub02():
    rfs = RFStation('ex-rfani/SC.LTA')
    rfs.moveoutcorrect()
    rfs.psrf2depth()
    rfs.psrf_1D_raytracing()
    rfs.slantstack()


def test_sub03():
    rfs = RFStation('ex-rfani/SC.LTA')
    rfs.resample(0.1)
    rfs.harmonic(-2, 12)
    rfs.harmo.write_constant()
    rfs.harmo.plot()


def test_sub04():
    rfs = RFStation('ex-rfani/SC.LTA')
    # rfs = RFStation('/Users/xumijian/Codes/seispy-example/ex-rfani/SC.LTA')
    rfs.resample(0.1)
    rfs.plotr(out_path='./', xlim=[-2, 40], key='rayp', enf=6, outformat='f')
    rfs.plotrt(out_path='./', xlim=[-2, 25], key='bazi', enf=3, outformat='g')


if __name__ == '__main__':
    test_sub04()


from os.path import join, dirname, exists
from scipy.io import loadmat
from matplotlib.colors import ListedColormap
import matplotlib.pyplot as plt
from seispy import geo
from seispy.geo import geo2sph, sph2geo
from seispy import distaz
import numpy as np
from scipy.interpolate import interp1d, interpn
import seispy


def vs2vprho(vs):
    vp = 0.9409 + 2.0947*vs - 0.8206*vs**2 + 0.2683*vs**3 - 0.0251*vs**4
    rho = 1.6612*vp - 0.4721*vp**2 + 0.0671*vp**3 - 0.0043*vp**4 + 0.000106*vp**5
    # vs = 0.7858 - 1.2344*vp + 0.7949*vp**2 - 0.1238*vp**3 + 0.0064*vp**4
    return vp, rho


def load_cyan_map():
    path = join(dirname(__file__), 'data', 'cyan.mat')
    carray = loadmat(path)['cyan']
    return ListedColormap(carray)


def check_path(key, path):
    if not exists(path):
        raise FileNotFoundError('No such file or directory of {}: {}'.format(key, path))
    else:
        return path

def check_stack_val(stack_val, dep_val):
    if np.mod(stack_val, dep_val) == 0:
        return stack_val/dep_val
    else:
        raise ValueError('stack_val must be a multiple of dep_val')


def read_rfdep(path):
    try:
        return np.load(path, allow_pickle=True)
    except:
        try:
            return np.load(path+'.npy', allow_pickle=True)
        except Exception as e:
            raise FileNotFoundError('Cannot open file of {}'.format(path))


def _from_layer_model(dep_range, h, vp, vs, rho=None):
    dep = 0
    vp_dep = np.zeros_like(dep_range).astype(float)
    vs_dep = np.zeros_like(dep_range).astype(float)
    rho_dep = np.zeros_like(dep_range).astype(float)
    for i, layer in enumerate(h):
        if (layer == 0 and i == len(h)-1) or \
           (dep+layer < dep_range[-1] and i == len(h)-1):
           idx = np.where(dep_range>=dep)[0]
        else:
            idx = np.where((dep_range >= dep) & (dep_range < dep+layer))[0]
        if idx.size == 0:
            raise ValueError('The thickness of layer {} less than the depth interval'.format(i+1))
        vp_dep[idx] = vp[i]
        vs_dep[idx] = vs[i]
        if rho is not None:
            rho_dep[idx] = rho[i]
        dep += layer
        if dep > dep_range[-1]:
            break
    return vp_dep, vs_dep, rho_dep


class DepModel(object):
    def __init__(self, dep_range, velmod='iasp91', elevation=0., layer_mod=False):
        """Class for computing back projection of Ps Ray paths.

        Parameters
        ----------
        dep_range : numpy.ndarray
            Depth range for conversion
        velmod : str, optional
            Text file of 1D velocity model with first 3 columns of depth/thickness, Vp and Vs,
            by default 'iasp91'
        elevation : float, optional
            Elevation in km, by default 0.0
        """
        self.isrho = False
        self.elevation = elevation
        self.layer_mod = layer_mod
        self.depths = dep_range.astype(float)
        try:
            self.model_array = np.loadtxt(self.from_file(velmod))
        except (ValueError, TypeError):
            return
        else:
            self.read_model_file()
            self.discretize()

    def read_model_file(self):
        self.dep_val = np.average(np.diff(self.depths))
        if self.layer_mod:
            self.depthsraw = self.depths
            if self.model_array.shape[1] == 4:
                self.isrho = True
                rho_array = self.model_array[:, 3]
            else:
                rho_array = None
            self.vpraw, self.vsraw, self.rhoraw = _from_layer_model(self.depths,
                                                       self.model_array[:, 0],
                                                       self.model_array[:, 1],
                                                       self.model_array[:, 2],
                                                       rho=rho_array
                                                       )
        else:
            self.depthsraw = self.model_array[:, 0]
            self.vpraw = self.model_array[:, 1]
            self.vsraw = self.model_array[:, 2]
            if self.model_array.shape[1] == 4:
                self.isrho = True
                self.rhoraw = self.model_array[:, 3]

    @classmethod
    def read_layer_model(cls, dep_range, h, vp, vs, rho=None, elevation=0):
        mod = cls(dep_range, velmod=None, layer_mod=True, elevation=elevation)
        if rho is not None:
            mod.isrho = True
        mod.depthsraw = mod.depths
        mod.vpraw, mod.vsraw, mod.rhoraw = _from_layer_model(mod.depths, h, vp, vs, rho=rho)
        mod.discretize()
        return mod

    def plot_model(self, show=True):
        plt.style.use('bmh')
        if self.isrho:
            self.model_fig = plt.figure(figsize=(6,6))
            fignum = 2
        else:
            self.model_fig = plt.figure(figsize=(4,6))
            fignum = 1
        self.model_ax = self.model_fig.add_subplot(1,fignum,1)
        self.model_ax.step(self.vp, self.depths, where='pre', label='Vp')
        self.model_ax.step(self.vs, self.depths, where='pre', label='Vs')
        self.model_ax.legend()
        self.model_ax.set_xlabel('Velocity (km/s)')
        self.model_ax.set_ylabel('Depth (km)')
        self.model_ax.set_ylim([self.depths[0], self.depths[-1]])
        self.model_ax.invert_yaxis()
        if self.isrho:
            self.rho_ax = self.model_fig.add_subplot(1,fignum,2)
            self.rho_ax.step(self.rho, self.depths, where='pre', color='C2', label='Density')
            self.rho_ax.legend()
            self.rho_ax.set_xlabel('Density (km/s)')
            self.rho_ax.set_ylim([self.depths[0], self.depths[-1]])
            self.rho_ax.invert_yaxis()
        if show:
            plt.show()

    def discretize(self):
        if self.elevation == 0:
            self.depths_elev = self.depths
            self.depths_extend = self.depths
        else:
            dep_append = np.arange(self.depths[-1]+self.dep_val, 
                               self.depths[-1]+self.dep_val+np.floor(self.elevation/self.dep_val+1), self.dep_val)
            self.depths_extend = np.append(self.depths, dep_append)
            self.depths_elev = np.append(self.depths, dep_append) - self.elevation
        self.dz = np.append(0, np.diff(self.depths_extend))
        self.thickness = np.append(np.diff(self.depths_extend), 0.)
        self.vp = interp1d(self.depthsraw, self.vpraw, bounds_error=False,
                           fill_value=self.vpraw[0])(self.depths_elev)
        self.vs = interp1d(self.depthsraw, self.vsraw, bounds_error=False,
                           fill_value=self.vsraw[0])(self.depths_elev)
        if self.isrho:
            self.rho = interp1d(self.depthsraw, self.rhoraw, bounds_error=False,
                               fill_value=self.rhoraw[0])(self.depths_elev)
        else:
            _, self.rho = vs2vprho(self.vs)
        self.R = 6371.0 - self.depths_elev

    def from_file(self, mode_name):
        if not isinstance(mode_name, str):
            raise TypeError('velmod should be in str type')
        if exists(mode_name):
            filename = mode_name
        elif exists(join(dirname(__file__), 'data', mode_name.lower()+'.vel')):
            filename = join(dirname(__file__), 'data', mode_name.lower()+'.vel')
        else:
            raise ValueError('No such file of velocity model')
        return filename

    def tpds(self, rayps, raypp, sphere=True):
        if sphere:
            radius = self.R
        else:
            radius = 6371.
        tps = np.cumsum((np.sqrt((radius / self.vs) ** 2 - rayps ** 2) -
                        np.sqrt((radius / self.vp) ** 2 - raypp ** 2)) *
                        (self.dz / radius))
        return tps
    
    def tpppds(self, rayps, raypp, sphere=True):
        if sphere:
            radius = self.R
        else:
            radius = 6371.
        tps = np.cumsum((np.sqrt((radius / self.vs) ** 2 - rayps ** 2) +
                        np.sqrt((radius / self.vp) ** 2 - raypp ** 2)) *
                        (self.dz / radius))
        return tps
    
    def tpspds(self, rayps, sphere=True):
        if sphere:
            radius = self.R
        else:
            radius = 6371.
        tps = np.cumsum(2*np.sqrt((radius / self.vs) ** 2 - rayps ** 2)*
                        (self.dz / radius))
        return tps

    def radius_s(self, rayp, phase='P', sphere=True):
        if phase == 'P':
            vel = self.vp
        else:
            vel = self.vs
        if sphere:
            radius = self.R
        else:
            radius = 6371.
        hor_dis = np.cumsum((self.dz / radius) / np.sqrt((1. / (rayp ** 2. * (radius / vel) ** -2)) - 1))
        return hor_dis

    def raylength(self, rayp, phase='P', sphere=True):
        if phase == 'P':
            vel = self.vp
        else:
            vel = self.vs
        if sphere:
            radius = self.R
        else:
            radius = 6371.
        raylen = (self.dz * radius) / (np.sqrt(((radius / self.vs) ** 2) - (rayp ** 2)) * vel)
        return raylen


class Mod3DPerturbation:
    def __init__(self, modpath, YAxisRange, velmod='iasp91'):
        dep_mod = DepModel(YAxisRange, velmod=velmod)
        self.model = np.load(modpath)
        new1dvp = interp1d(dep_mod.depthsraw, dep_mod.vpraw)(self.model['dep'])
        new1dvs = interp1d(dep_mod.depthsraw, dep_mod.vsraw)(self.model['dep'])
        new1dvp, _, _ = np.meshgrid(new1dvp, self.model['lat'], self.model['lon'], indexing='ij')
        new1dvs, _, _ = np.meshgrid(new1dvs, self.model['lat'], self.model['lon'], indexing='ij')
        self.dvp = (self.model['vp'] - new1dvp) / new1dvp
        self.dvs = (self.model['vs'] - new1dvs) / new1dvs
        self.cvp = dep_mod.vp
        self.cvs = dep_mod.vs

    def interpdvp(self, points):
        dvp = interpn((self.model['dep'], self.model['lat'], self.model['lon']), self.dvp, points,
                      bounds_error=False, fill_value=None)
        return dvp

    def interpdvs(self, points):
        dvs = interpn((self.model['dep'], self.model['lat'], self.model['lon']), self.dvs, points,
                      bounds_error=False, fill_value=None)
        return dvs


def create_center_bin_profile(stations, val=5, method='linear'):
    if not isinstance(stations, seispy.rf2depth_makedata.Station):
        raise TypeError('Stations should be seispy.rf2depth_makedata.Station')
    dis_sta = prof_range(stations.stla, stations.stlo)
    dis_inter = np.append(np.arange(0, dis_sta[-1], val), dis_sta[-1])
    r, theta, phi = geo2sph(np.zeros(stations.stla.size), stations.stla, stations.stlo)
    # t_po = np.arange(stations.stla.size)
    # ip_t_po = np.linspace(0, stations.stla.size, bin_num)
    theta_i = interp1d(dis_sta, theta, kind=method, bounds_error=False, fill_value='extrapolate')(dis_inter)
    phi_i = interp1d(dis_sta, phi, kind=method, bounds_error=False, fill_value='extrapolate')(dis_inter)
    _, lat, lon = sph2geo(r, theta_i, phi_i)
    # dis = prof_range(lat, lon)
    return lat, lon, dis_inter


def prof_range(lat, lon):
    dis = [0]
    for i in range(lat.size-1):
        dis.append(distaz(lat[i], lon[i], lat[i+1], lon[i+1]).degreesToKilometers())
    return np.cumsum(dis)


# matplotlib.use("Qt5Agg")
from PySide6.QtCore import Qt
from PySide6.QtGui import QIcon, QKeySequence, QGuiApplication, QShortcut, QAction
from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, \
                            QSizePolicy, QWidget, \
                            QPushButton, QHBoxLayout, QFileDialog
from os.path import exists, dirname, join
from seispy.setuplog import setuplog
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd


class SFigure(Figure):
    def __init__(self, eqs, para, logger, width=21, height=11, dpi=100):
        self.eqs = eqs
        self.logger = logger
        self.row_num = eqs.shape[0]
        self.picker_time = pd.DataFrame({'trigger_shift': np.array([eqs.iloc[i]['data'].trigger_shift for i in range(self.row_num)])})
        self.picker_time.set_index(eqs.index, inplace=True)
        self.para = para
        self.log = setuplog()
        self.idx = 0
        self.init_figure(width=width, height=height, dpi=dpi)
        self.plot()
        self.drop_lst = []
        self.drop_color = 'lightgray'

    def init_figure(self, width=21, height=11, dpi=100):
        self.fig, self.axs = plt.subplots(3, 1, sharex=True, figsize=(width, height), dpi=dpi, tight_layout=True)

    def set_cross_hair_visible(self, visible):
        need_redraw = self.cursor[0].get_visible() != visible
        # self.horizontal_line.set_visible(visible)
        for cursor in self.cursor:
            cursor.set_visible(visible)
        # self.text.set_visible(visible)
        return need_redraw
    
    def set_properties(self):
        date = self.eqs.iloc[self.idx]['date'].strftime("%Y.%m.%dT%H:%M:%S")
        lat = self.eqs.iloc[self.idx]['evla']
        lon = self.eqs.iloc[self.idx]['evlo']
        dep = self.eqs.iloc[self.idx]['evdp']
        dis = self.eqs.iloc[self.idx]['dis']
        bazi = self.eqs.iloc[self.idx]['bazi']
        mag = self.eqs.iloc[self.idx]['mag']
        title = '({}/{}) {}, Lat: {:.2f}, Lon: {:.2f}, Depth: {:.1f}, \nBaz: {:.2f}$^{{\circ}}$, Distance:{:.2f} km, Mw: {:.1f}'.format(
                self.idx+1, self.row_num, date, lat, lon, dep, bazi, dis, mag)
        self.fig.suptitle(title)
        for ax in self.axs:
            ax.set(xlim=[-self.para.time_before, self.para.time_after])
            ax.autoscale(enable=True,axis='y', tight=True)
            ax.minorticks_on()
        self.axs[2].set_xlabel('Time (s)')
        self.axs[0].legend(loc=2)


    def clear(self):
        for ax in self.axs:
            ax.cla()

    def _plot_line(self, st, lc='tab:blue'):
        self.picker = []
        self.lines = []
        for i in range(3):
            self.lines.append(self.axs[i].plot(self.time_axis, st[i].data, color=lc))
            self.axs[i].axvline(x=0, lw=1.5, color='r', label='Theoretical')
            self.picker.append(self.axs[i].axvline(self.picker_time.iloc[self.idx]['trigger_shift'],
                                                   label='Triggered', color='k', lw=1.5, visible=True))
            self.axs[i].set_ylabel(st[i].stats.channel)

    def plot(self, lc='tab:blue'):
        st = self.eqs.iloc[self.idx]['data'].st_pick
        if len(st) == 0:
            return
        # t1, t2 = self.eqs.iloc[self.idx]['data'].t1_pick, st = self.eqs.iloc[self.idx]['data'].t2_pick
        self.time_axis = st[0].times()-self.para.time_before
        self._plot_line(st, lc)
        self.set_properties()

    def next_action(self):
        self.clear()
        self.idx += 1
        if self.idx >= self.row_num:
            self.idx = 0
        if self.eqs.index[self.idx] in self.drop_lst:
            self.plot(lc=self.drop_color)
        else:
            self.plot()
    
    def on_mouse_move(self, event):
        if not event.inaxes:
            need_redraw = self.set_cross_hair_visible(False)
            if need_redraw:
                for ax in self.axs:
                    ax.figure.canvas.draw()
        else:
            self.set_cross_hair_visible(True)
            x = event.xdata
            # update the line positions
            # self.horizontal_line.set_ydata(y)
            for cursor in self.cursor:
                cursor.set_xdata(x)
                # ax.figure.canvas.draw()
            # self.cursor[1].set_xdata(x)
            # self.axs[1].figure.canvas.draw()
        
    def on_click(self, event):
        if not event.inaxes:
            return
        if self.eqs.index[self.idx] in self.drop_lst:
            return
        self.picker_time.iloc[self.idx]['trigger_shift'] = event.xdata
        for picker in self.picker:
            picker.set_xdata(self.picker_time.iloc[self.idx]['trigger_shift'])
            picker.set_visible(True)

    def back_action(self):
        self.clear()
        self.idx -= 1
        if self.idx < 0 :
            self.idx = 0
        if self.eqs.index[self.idx] in self.drop_lst:
            self.plot(lc=self.drop_color)
        else:
            self.plot()

    def drop(self):
        if self.eqs.index[self.idx] in self.drop_lst:
            return
        self.drop_lst.append(self.eqs.index[self.idx])
        self.logger.RFlog.info('{} is marked'.format(self.eqs.loc[self.eqs.index[self.idx], 'datestr']))
        # self.clear()
        for line in self.lines:
            line[0].set_color(self.drop_color)
        # self._plot_line(self.eqs.iloc[self.idx]['data'].st_pick, )

    def cancel(self):
        if self.eqs.index[self.idx] not in self.drop_lst:
            return
        self.drop_lst.remove(self.eqs.index[self.idx])
        self.logger.RFlog.info('{} is unmarked'.format(self.eqs.loc[self.eqs.index[self.idx], 'datestr']))
        # self.clear()
        for line in self.lines:
            line[0].set_color('tab:blue')
        # self._plot_line(self.eqs.iloc[self.idx]['data'].st_pick)

    def finish(self):
        self.eqs.drop(self.drop_lst, inplace=True)
        for i, row in self.eqs.iterrows():
            row['data'].trigger_shift = self.picker_time['trigger_shift'][i]

class MyMplCanvas(FigureCanvas):
    def __init__(self, parent=None, eqs=None, para=None, logger=None, width=21, height=11, dpi=100):
        plt.rcParams['axes.unicode_minus'] = False 
        self.sfig = SFigure(eqs, para, logger, width=width, height=height, dpi=dpi)
        FigureCanvas.__init__(self, self.sfig.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Policy.Expanding,
                                   QSizePolicy.Policy.Expanding)
        FigureCanvas.updateGeometry(self)
        FigureCanvas.setCursor(self, Qt.CursorShape.CrossCursor)

class MatplotlibWidget(QMainWindow):
    def __init__(self, eqs, para, logger, parent=None):
        super(MatplotlibWidget, self).__init__(parent)
        self.initUi(eqs, para, logger)

    def initUi(self, eqs, para, logger):
        self.mpl = MyMplCanvas(self, eqs=eqs, para=para, logger=logger, width=8, height=4, dpi=50)
        # self.mpl.mpl_connect('motion_notify_event', self.on_mouse_move)
        self.mpl.mpl_connect('button_press_event', self.on_click)
        self.layout = QVBoxLayout()
        self.add_btn()
        self.layout.addWidget(self.mpl, 2)

        main_frame = QWidget()
        self.setCentralWidget(main_frame)
        main_frame.setLayout(self.layout)

        self._set_geom_center()
        self._define_global_shortcuts()
        self.setWindowTitle('Pick S Phase')
        self.setWindowIcon(QIcon(join(dirname(__file__), 'data', 'seispy.png')))

    def add_btn(self):
        pre_btn = QPushButton("Back (z)")
        # pre_btn.setCursor(Qt.ArrowCursor)
        pre_btn.clicked.connect(self.back_connect)
        next_btn = QPushButton("Next (c)")
        # next_btn.setCursor(Qt.ArrowCursor)
        next_btn.clicked.connect(self.next_connect)
        finish_btn = QPushButton("Finish (Enter)")
        # finish_btn.setCursor(Qt.ArrowCursor)
        finish_btn.clicked.connect(self.finish_connect)
        btnbox = QHBoxLayout()
        btnbox.addStretch(1)
        btnbox.addWidget(pre_btn)
        btnbox.addWidget(next_btn)
        btnbox.addWidget(finish_btn)

        self.mark_btn = QPushButton("Mark to poor (d)")
        # mark_btn.setCursor(Qt.ArrowCursor)
        self.mark_btn.setCheckable(True)
        self.mark_btn.clicked.connect(self.drop_connect)
        pathbox = QHBoxLayout()
        pathbox.addWidget(self.mark_btn)

        ctrl_layout = QHBoxLayout()
        ctrl_layout.addLayout(pathbox)
        ctrl_layout.addLayout(btnbox)

        self.layout.addLayout(ctrl_layout)

    # def mark_btn_connect(self, mark_btn):
    #     if mark_btn.isChecked():
    #         self.drop_connect()
    #     else:
    #         self.cancel_connect()

    def exit_app(self):
        self.close()

    def on_click(self, event):
        self.mpl.sfig.on_click(event)
        self.mpl.draw()
    
    def on_mouse_move(self, event):
        self.mpl.sfig.on_mouse_move(event)
        self.mpl.draw()

    def check_mark_btn(self):
        sfig = self.mpl.sfig
        if sfig.eqs.index[sfig.idx] in sfig.drop_lst:
            self.mark_btn.setChecked(True)
        else:
            self.mark_btn.setChecked(False)

    def check_drop(self):
        sfig = self.mpl.sfig
        if sfig.eqs.index[sfig.idx] in sfig.drop_lst:
            self.mpl.sfig.cancel()
            self.mark_btn.setChecked(False)
        else:
            self.mpl.sfig.drop()
            self.mark_btn.setChecked(True)

    def next_connect(self):
        self.mpl.sfig.next_action()
        self.mpl.draw()
        self.check_mark_btn()
    
    def back_connect(self):
        self.mpl.sfig.back_action()
        self.mpl.draw()
        self.check_mark_btn()
    
    def drop_connect(self):
        # self.mark_btn.setChecked(True)
        # self.mpl.sfig.drop()
        self.check_drop()
        self.mpl.draw()
    
    # def cancel_connect(self):
    #     self.mark_btn.setChecked(False)
    #     self.mpl.sfig.cancel()
    #     self.mpl.draw()
    
    def finish_connect(self):
        self.mpl.sfig.finish()
        self.close()
        # QApplication.quit()
    
    def _define_global_shortcuts(self):
        self.key_c = QShortcut(QKeySequence('c'), self)
        self.key_c.activated.connect(self.next_connect)
        self.key_z = QShortcut(QKeySequence('z'), self)
        self.key_z.activated.connect(self.back_connect)
        self.key_d = QShortcut(QKeySequence('d'), self)
        self.key_d.activated.connect(self.drop_connect)
        # self.key_a = QShortcut(QKeySequence('a'), self)
        # self.key_a.activated.connect(self.cancel_connect)
        self.key_enter = QShortcut(QKeySequence('Return'), self)
        self.key_enter.activated.connect(self.finish_connect)

    def _set_geom_center(self, height=0.7, width=1):
        screen_resolution = QGuiApplication.primaryScreen().geometry()
        screen_height = screen_resolution.height()
        screen_width = screen_resolution.width()
        frame_height = int(screen_height * height)
        frame_width = int(screen_width * width)

        self.setGeometry(0, 0, frame_width, frame_height)
        self.move((screen_width / 2) - (self.frameSize().width() / 2),
                  (screen_height / 2) - (self.frameSize().height() / 2))


import obspy
from obspy import UTCDateTime
from obspy.taup import TauPyModel
from obspy.io.sac import SACTrace
from obspy.core.event.catalog import read_events
import re
from os.path import join, exists
from os import makedirs
from seispy.io import Query, _cat2df
from seispy.para import RFPara
from seispy import distaz
from seispy.eq import EQ
from seispy.setuplog import setuplog
from seispy.catalog import read_catalog_file
import glob
import numpy as np
from datetime import timedelta
import pandas as pd
import configparser
import argparse
import sys
import pickle


def pickphase(eqs, para, logger):
    from PySide6.QtWidgets import QApplication
    from seispy.pickseis.sviewerui import MatplotlibWidget
    app = QApplication(sys.argv)
    ui = MatplotlibWidget(eqs, para, logger)
    ui.show()
    if app.exec() == 0:
        ui.exit_app()
        return


def _add_header(st, evt_time, stainfo):
    sta = stainfo.query.stations[0][0]
    for tr in st:
        header = SACTrace.from_obspy_trace(tr).to_obspy_trace().stats.sac
        channel = tr.stats.channel
        for ch in sta.channels:
            if ch.code == channel:
                header.cmpaz = ch.azimuth
                header.cmpinc = -ch.dip
        header.stla = stainfo.stla
        header.stlo = stainfo.stlo
        header.stel = stainfo.stel
        header.b = 0
        header.o = evt_time - tr.stats.starttime
        tr.stats.__dict__['sac'] = header


class SACFileNotFoundError(Exception):
    def __init__(self, matchkey):
        self.matchkey = matchkey
    def __str__(self):
        print('No sac files found with {}'.format(self.matchkey))


def datestr2regex(datestr):
    pattern = datestr.replace('%y', r'\d{2}')
    pattern = pattern.replace('%Y', r'\d{4}')
    pattern = pattern.replace('%m', r'\d{2}')
    pattern = pattern.replace('%d', r'\d{2}')
    pattern = pattern.replace('%j', r'\d{3}')
    pattern = pattern.replace('%H', r'\d{2}')
    pattern = pattern.replace('%M', r'\d{2}')
    pattern = pattern.replace('%S', r'\d{2}')
    return pattern


def read_catalog(logpath:str, b_time, e_time, stla:float, stlo:float,
                 magmin=5.5, magmax=10., dismin=30., dismax=90.):
    """Read local catalog with seispy or QUAKEML format

    :param logpath: Path to catalogs
    :type logpath: str
    :param b_time: Start time 
    :type b_time: obspy.UTCDateTime
    :param e_time: End time
    :type e_time: obspy.UTCDateTime
    :param stla: Station latitude
    :type stla: float
    :param stlo: Station longitude
    :type stlo: float
    :param magmin: Minimum magnitude, defaults to 5.5
    :type magmin: float, optional
    :param magmax: Maximum magnitude, defaults to 10
    :type magmax: float, optional
    :param dismin: Minimum distance, defaults to 30
    :type dismin: float, optional
    :param dismax: Maximum distance, defaults to 90
    :type dismax: float, optional
    :return: list of earthquakes
    :rtype: pandas.DataFrame
    """
    try:
        eq_lst = read_catalog_file(logpath)
    except:
        events = read_events(logpath, 'QUAKEML')
        eq_lst = _cat2df(events)
    dis = distaz(stla, stlo, eq_lst['evla'], eq_lst['evlo']).delta
    eq_lst = eq_lst[(eq_lst['date']>=b_time) & (eq_lst['date']<=e_time) & \
                    (eq_lst['mag']>=magmin) & (eq_lst['mag']<=magmax) & \
                    (dis>=dismin) & (dis<=dismax)]
    return eq_lst


def fetch_waveform(eq_lst, para, model, logger):
    tb = np.max([2*para.noiselen, 2*para.time_before])
    te = np.max([2*para.noiselen, 2*para.time_after])
    query = Query(para.data_server)
    new_col = ['dis', 'bazi', 'data', 'datestr']
    eq_match = pd.DataFrame(columns=new_col)
    for i, row in eq_lst.iterrows():
        datestr = row['date'].strftime('%Y.%j.%H.%M.%S')
        daz = distaz(para.stainfo.stla, para.stainfo.stlo, row['evla'], row['evlo'])
        arrivals = model.get_travel_times(row['evdp'], daz.delta, phase_list=[para.phase])
        if not arrivals:
            logger.RFlog.error('The phase of {} with source depth {} and distance {} is not exists'.format(
                                     para.phase, row['evdp'], daz.delta))
            continue
        if len(arrivals) > 1:
            logger.RFlog.error('More than one phase were calculated with source depth of {} and distance of {}'.format(
                               row['evdp'], daz.delta))
        else:
            arr_time = arrivals[0].time
        t1 = row['date']+arr_time-tb
        t2 = row['date']+arr_time+te
        try:
            logger.RFlog.info('Fetch waveforms of ({}/{}) event {} from {}'.format(
                              i+1, eq_lst.shape[0], datestr, para.data_server))
            st = query.client.get_waveforms(para.stainfo.network, para.stainfo.station,
                                            para.stainfo.location, para.stainfo.channel, t1, t2)
            _add_header(st, row['date'], para.stainfo)
        except Exception as e:
            logger.RFlog.error('Error in fetching waveforms of event {}: {}'.format(datestr, str(e).strip()))
            continue
        try:
            this_eq = EQ.from_stream(st)
        except Exception as e:
            logger.RFlog.error('{}'.format(e))
            continue
        this_eq.get_time_offset(row['date'])
        this_df = pd.DataFrame([[daz.delta, daz.baz, this_eq, datestr]], columns=new_col, index=[i])
        eq_match = pd.concat([eq_match, this_df])
    ind = eq_match.index.drop_duplicates(keep=False)
    eq_match = eq_match.loc[ind]
    return pd.concat([eq_lst, eq_match], axis=1, join='inner')


def match_eq(eq_lst, pathname, stla, stlo, logger, ref_comp='Z', suffix='SAC', offset=None,
             tolerance=210, dateformat='%Y.%j.%H.%M.%S'):
    pattern = datestr2regex(dateformat)
    ref_eqs = glob.glob(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
    if len(ref_eqs) == 0:
        raise SACFileNotFoundError(join(pathname, '*{0}*{1}'.format(ref_comp, suffix)))
    sac_files = []
    for ref_sac in ref_eqs:
        try:
            datestr = re.findall(pattern, ref_sac)[0]
        except IndexError:
            raise IndexError('Error data format of {} in {}'.format(pattern, ref_sac))
        if isinstance(offset, (int, float)):
            sac_files.append([datestr, UTCDateTime.strptime(datestr, dateformat), -offset])
        elif offset is None:
            try:
                tr = obspy.read(ref_sac)[0]
            except TypeError:
                continue
            sac_files.append([datestr, tr.stats.starttime-tr.stats.sac.b, tr.stats.sac.o])
        else:
            raise TypeError('offset should be int or float type')
    new_col = ['dis', 'bazi', 'data', 'datestr']
    eq_match = pd.DataFrame(columns=new_col)
    for datestr, b_time, offs in sac_files:
        date_range_begin = b_time + timedelta(seconds=offs - tolerance)
        date_range_end = b_time + timedelta(seconds=offs + tolerance)
        results = eq_lst[(eq_lst.date > date_range_begin) & (eq_lst.date < date_range_end)]
        if len(results) != 1:
            continue
        try:
            this_eq = EQ(pathname, datestr, suffix)
        except Exception as e:
            logger.RFlog.error('{}'.format(e))
            continue
        this_eq.get_time_offset(results.iloc[0]['date'])
        daz = distaz(stla, stlo, results.iloc[0]['evla'], results.iloc[0]['evlo'])
        this_df = pd.DataFrame([[daz.delta, daz.baz, this_eq, datestr]], columns=new_col, index=results.index.values)
        eq_match = pd.concat([eq_match, this_df])
    ind = eq_match.index.drop_duplicates(keep=False)
    eq_match = eq_match.loc[ind]
    return pd.concat([eq_lst, eq_match], axis=1, join='inner')


def CfgModify(cfg_file, session, key, value):
    cf = configparser.RawConfigParser()
    try:
        cf.read(cfg_file)
    except Exception:
        raise FileNotFoundError('Cannot open configure file %s' % cfg_file)
    cf.set(session, key, value)
    cf.write(open(cfg_file, 'w'))


class RF(object):
    def __init__(self, cfg_file=None, log=None):
        if log is None:
            self.logger = setuplog()
        else:
            self.logger = log
        if cfg_file is None:
            self.para = RFPara()
        elif isinstance(cfg_file, str):
            if not exists(cfg_file):
                self.logger.RFlog.error('No such file of {}.'.format(cfg_file))
                sys.exit(1)
            self.para = RFPara.read_para(cfg_file)
        else:
            raise TypeError('cfg should be in \'str\' format rather than \'{0}\''.format(type(cfg_file)))
        if not isinstance(self.para, RFPara):
            raise TypeError('Input value should be class seispy.rf.para')
        self.eq_lst = pd.DataFrame()
        self.eqs = pd.DataFrame()
        self.model = TauPyModel(self.para.velmod)
        self.stainfo = self.para.stainfo
        self.baz_shift = 0

    @property
    def date_begin(self):
        return self.para.date_begin

    @date_begin.setter
    def date_begin(self, value):
        self.para.date_begin = value

    @property
    def date_end(self):
        return self.para.date_end

    @date_end.setter
    def date_end(self, value):
        self.para.date_end = value

    def load_stainfo(self):
        try:
            if self.para.use_remote_data:
                self.logger.RFlog.info('Load station info of {}.{} from {} web-service'.format(
                    self.para.stainfo.network, self.para.stainfo.station, self.para.data_server))
                self.para.stainfo.get_station_from_ws(self.para.data_server)
                try:
                    self.para._check_date_range()
                except Exception as e:
                    self.logger.RFlog.error('{}'.format(e))
                    sys.exit(1)
            else:
                self.logger.RFlog.info('Load station info from {0}'.format(self.para.datapath))
                self.para.stainfo.load_stainfo(self.para.datapath, self.para.ref_comp, self.para.suffix)
            self.logger.RFlog.info('{}/{}, latitude: {:.3f}, longiture: {:.3f}'.format(
                self.para.stainfo.network, self.para.stainfo.station, self.stainfo.stla, self.stainfo.stlo))
        except Exception as e:
            self.logger.RFlog.error('Error in loading station info: {0}'.format(e))
            sys.exit(1)

    def search_eq(self, local=False, catalog=None):
        if not local:
            try:
                self.logger.RFlog.info('Searching earthquakes from {}'.format(self.para.cata_server))
                if self.para.date_end > UTCDateTime():
                    self.para.date_end = UTCDateTime()
                query = Query(self.para.cata_server)
                query.get_events(starttime=self.para.date_begin, endtime=self.para.date_end,
                                 latitude=self.para.stainfo.stla, longitude=self.para.stainfo.stlo,
                                 minmagnitude=self.para.magmin, maxmagnitude=self.para.magmax,
                                 minradius=self.para.dismin, maxradius=self.para.dismax, catalog=catalog)
                self.eq_lst = query.events
            except Exception as e:
                raise ConnectionError(e)
        else:
            try:
                self.logger.RFlog.info(
                    'Searching earthquakes from {0} to {1}'.format(self.date_begin.strftime('%Y.%m.%dT%H:%M:%S'),
                                                                   self.date_end.strftime('%Y.%m.%dT%H:%M:%S')))
                self.eq_lst = read_catalog(self.para.catalogpath, self.para.date_begin, self.para.date_end,
                                           self.para.stainfo.stla, self.para.stainfo.stlo,
                                           magmin=self.para.magmin, magmax=self.para.magmax,
                                           dismin=self.para.dismin, dismax=self.para.dismax)
            except Exception as e:
                self.logger.RFlog.error('{0}'.format(e))
                raise e
        self.logger.RFlog.info('{} earthquakes are found'.format(self.eq_lst.shape[0]))

    def match_eq(self):
        try:
            if self.para.use_remote_data:
                self.logger.RFlog.info('Fetch seismic data from {}'.format(self.para.data_server))
                self.eqs = fetch_waveform(self.eq_lst, self.para, self.model, self.logger)
            else:
                self.logger.RFlog.info('Associating SAC files with earthquakes')
                self.eqs = match_eq(self.eq_lst, self.para.datapath, self.para.stainfo.stla, 
                                    self.para.stainfo.stlo, self.logger,
                                    ref_comp=self.para.ref_comp, suffix=self.para.suffix,
                                    offset=self.para.offset, tolerance=self.para.tolerance,
                                    dateformat=self.para.dateformat)
        except Exception as e:
            self.logger.RFlog.error('{0}'.format(e))
            raise e
        if self.eqs.shape[0] == 0:
            self.logger.RFlog.warning('No earthquakes associated, please check configurations.'.format(self.eqs.shape[0]))
            sys.exit(1)
        else:
            self.logger.RFlog.info('{0} earthquakes are associated'.format(self.eqs.shape[0]))

    def save_raw_data(self):
        if not exists(self.para.datapath):
            makedirs(self.para.datapath)
        for i, row in self.eqs.iterrows():
            row['data'].write(self.para.datapath, row['date'])

    def savepjt(self):
        eqs = self.eqs.copy()
        for _, row in eqs.iterrows():
            row['data'].cleanstream()
        try:
            self.logger.RFlog.info('Saving project to {0}'.format(self.para.pjtpath))
            with open(self.para.pjtpath, 'wb') as f:
                pickle.dump({'para': self.para, 'eqs': eqs}, f, -1)
        except Exception as e:
            self.logger.RFlog.error('{0}'.format(e))
            raise IOError(e)
 
    @classmethod
    def loadpjt(cls, path):
        with open(path, 'rb') as f:
            rfdata = pickle.load(f)
        pjt = cls(phase=rfdata['para'].phase)
        pjt.para = rfdata['para']
        if not exists(pjt.para.datapath):
            pjt.logger.RFlog.error('Data path {} was not found'.format(pjt.para.datapath))
            sys.exit(1)
        pjt.load_stainfo()
        pjt.eqs = rfdata['eqs']
        for _, row in pjt.eqs.iterrows():
            try:
                row['data'].readstream()
            except Exception as e:
                pjt.logger.RFlog.warning('Cannot read {}, skipping'.format(row['data'].filestr))
                continue
        return pjt

    def channel_correct(self):
        if self.para.switchEN or self.para.reverseN or self.para.reverseE:
            self.logger.RFlog.info('Correct components with switchEN: {}, reverseE: {}, reverseN: {}'.format(
                                    self.para.switchEN, self.para.reverseE, self.para.reverseN))
            for _, row in self.eqs.iterrows():
                row['data'].channel_correct(self.para.switchEN, self.para.reverseE, self.para.reverseN)
        
    def detrend(self):
        self.logger.RFlog.info('Detrend all data')
        drop_idx = []
        for i, row in self.eqs.iterrows():
            try:
                row['data'].detrend()
            except:
                self.logger.RFlog.error('Data error in {}'.format(row['date'].strftime('%Y.%j.%H.%M.%S')))
                drop_idx.append(i)
        self.eqs.drop(drop_idx, inplace=True)

    def filter(self, freqmin=None, freqmax=None, order=4):
        if freqmin is None:
            freqmin = self.para.freqmin
        if freqmax is None:
            freqmax = self.para.freqmax
        self.logger.RFlog.info('Filter all data from {0} to {1}'.format(freqmin, freqmax))
        for _, row in self.eqs.iterrows():
            row['data'].filter(freqmin=freqmin, freqmax=freqmax, order=order)

    def cal_phase(self):
        self.logger.RFlog.info('Calculate {} arrivals and ray parameters for all data'.format(self.para.phase))
        for _, row in self.eqs.iterrows():
            row['data'].get_arrival(self.model, row['evdp'], row['dis'], phase=self.para.phase)

    def baz_correct(self, time_b=10, time_e=20, offset=90, correct_angle=None):
        if correct_angle is not None:
            self.logger.RFlog.info('correct back-azimuth with {} deg.'.format(correct_angle))
            self.eqs['bazi'] = np.mod(self.eqs['bazi'] + correct_angle, 360)
        else:
            self.logger.RFlog.info('correct back-azimuth with T energy minimization')
            y = np.arange(self.eqs.shape[0])
            shift_all = np.array([])
            x = np.arange(-offset, offset)
            ampt_all = np.empty([0, x.shape[0]])
            for i, row in self.eqs.iterrows():
                curr_baz, ampt = row['data'].search_baz(row['bazi'], time_b=time_b, time_e=time_e, offset=offset)
                shift_all = np.append(shift_all, curr_baz)
                ampt_all = np.vstack((ampt_all, ampt))
            if None in shift_all:
                self.logger.RFlog.error('Range of searching bazi is too small.')
                sys.exit(1)
            self.baz_shift = np.mean(shift_all[np.where(np.logical_not(np.isnan(shift_all)))])
            self.logger.RFlog.info('Average {:.1f} deg offset in back-azimuth'.format(self.baz_shift))

    def rotate(self, search_inc=False):
        targ_comp = ''.join(sorted(self.para.comp.upper()))
        if targ_comp == 'RTZ':
            method='NE->RT'
        elif targ_comp == 'LQT':
            method='ZNE->LQT'
        else:
            raise ValueError('comp must be in RTZ or LQT.')
        self.logger.RFlog.info('Rotate {0} phase to {1}'.format(self.para.phase, method))
        drop_idx = []
        for i, row in self.eqs.iterrows():
            try:
                row['data'].rotate(row['bazi'], method=method, search_inc=search_inc, baz_shift=self.baz_shift)
            except Exception as e:
                self.logger.RFlog.error('{}: {}'.format(row['data'].datestr, e))
                drop_idx.append(i)
                continue
            if search_inc:
                self.logger.RFlog.info('The incidence angle of {} was corrected by {:.1f} deg'.format(
                                       row['data'].datestr, row['data'].inc_correction))
        self.eqs.drop(drop_idx, inplace=True)

    def drop_eq_snr(self, length=None):
        if length is None:
            length = self.para.noiselen
        self.logger.RFlog.info('Reject data record with SNR less than {0}'.format(self.para.noisegate))
        drop_lst = []
        for i, row in self.eqs.iterrows():
            snr_E, snr_N, snr_Z = row['data'].snr(length=length)
            mean_snr = np.mean([snr_E, snr_N, snr_Z])
            if mean_snr < self.para.noisegate:
                drop_lst.append(i)
        self.eqs.drop(drop_lst, inplace=True)
        self.logger.RFlog.info('{0} events left after SNR calculation'.format(self.eqs.shape[0]))

    def trim(self):
        self.logger.RFlog.info('Trim waveforms from {0:.2f} before {2} to {1:.2f} after {2}'.format(
                               self.para.time_before, self.para.time_after, self.para.phase))
        for _, row in self.eqs.iterrows():
            row['data'].trim(self.para.time_before, self.para.time_after)
    
    def pick(self, prepick=True, stl=5, ltl=10):
        if prepick:
            self.logger.RFlog.info('Pre-pick {} arrival using STA/LTA method'.format(self.para.phase))
            for _, row in self.eqs.iterrows():
                row['data'].phase_trigger(self.para.time_before, self.para.time_after,
                                          stl=stl, ltl=ltl)
        self.logger.RFlog.info('{0} events left after virtual checking'.format(self.eqs.shape[0]))
        pickphase(self.eqs, self.para, self.logger)

    def deconv(self):
        shift = self.para.time_before
        time_after = self.para.time_after
        drop_lst = []

        count = 0
        for i, row in self.eqs.iterrows():
            count += 1
            try:
                row['data'].deconvolute(shift, time_after, method=self.para.decon_method, f0=self.para.gauss,
                                        only_r=self.para.only_r, itmax=self.para.itmax, minderr=self.para.minderr,
                                        wlevel=self.para.wlevel, target_dt=self.para.target_dt)
                if self.para.decon_method == 'iter':
                    self.logger.RFlog.info('Iterative Decon {0} ({3}/{4}) iterations: {1}; final RMS: {2:.4f}'.format(
                        row['data'].datestr, row['data'].rf[0].stats.iter,
                        row['data'].rf[0].stats.rms[-1], count, self.eqs.shape[0]))
                elif self.para.decon_method == 'water':
                    self.logger.RFlog.info('Water level Decon {} ({}/{}); RMS: {:.4f}'.format(
                        row['data'].datestr, count, self.eqs.shape[0], row['data'].rf[0].stats.rms))
            except Exception as e:
                self.logger.RFlog.error('{}: {}'.format(row['data'].datestr, e))
                drop_lst.append(i)
        self.eqs.drop(drop_lst, inplace=True)

    def saverf(self, gauss=None):
        npts = int((self.para.time_before + self.para.time_after)/self.para.target_dt+1)
        if self.para.phase[-1] == 'P':
            shift = self.para.time_before
        elif self.para.phase[-1] == 'S':
            shift = self.para.time_after
        else:
            pass
        good_lst = []
        if isinstance(self.para.gauss, (int, float)):
            gauss = self.para.gauss
        elif gauss is None:
            gauss = self.para.gauss[0]
        else:
            if gauss in self.para.gauss:
                pass
            else:
                raise ValueError('gauss should be a element in the seispy.para.RFPara.gauss')

        if self.para.rmsgate is not None:
            self.logger.RFlog.info('Save RFs with final RMS less than {:.2f} and criterion of {}'.format(self.para.rmsgate, self.para.criterion))
        else:
            self.logger.RFlog.info('Save RFs with and criterion of {}'.format(self.para.criterion))
        for i, row in self.eqs.iterrows():
            if row['data'].judge_rf(gauss, shift, npts, criterion=self.para.criterion, rmsgate=self.para.rmsgate):
                row['data'].saverf(self.para.rfpath, evtstr=row['date'].strftime('%Y.%j.%H.%M.%S'), shift=shift,
                                   evla=row['evla'], evlo=row['evlo'], evdp=row['evdp'], baz=row['bazi'],
                                   mag=row['mag'], gcarc=row['dis'], gauss=gauss, only_r=self.para.only_r,
                                   user9=self.baz_shift)
                good_lst.append(i)
        self.logger.RFlog.info('{} PRFs are saved.'.format(len(good_lst)))
        self.eqs = self.eqs.loc[good_lst]


def setpar():
    parser = argparse.ArgumentParser(description="Set parameters to configure file")
    parser.add_argument('cfg_file', type=str, help='Path to configure file')
    parser.add_argument('session', type=str, help='session name')
    parser.add_argument('key', type=str, help='key name')
    parser.add_argument('value', type=str, help='value')
    arg = parser.parse_args()
    CfgModify(arg.cfg_file, arg.session, arg.key, arg.value)


if __name__ == '__main__':
    pass
    # get_events_test()
    # prf()
    # proj_test()



import os
from seispy.rf import RF
from obspy import UTCDateTime
import pytest
import glob

def test_sub01():
    rf = RF()
    rf.para.datapath = './data'
    rf.para.phase = 'P'
    rf.para.stainfo.network = 'IC'
    rf.para.stainfo.station = 'BJT'
    rf.para.stainfo.channel = 'HH?'
    rf.para.magmin = 6.5
    rf.para.gauss = [1.0, 1.5]
    rf.para.use_remote_data = True
    rf.load_stainfo()

    rf.para.date_begin = UTCDateTime('20220101')
    rf.para.date_end = UTCDateTime('20220301')
    rf.search_eq()
    rf.match_eq()
    rf.save_raw_data()
    rf.detrend()
    rf.filter()
    rf.cal_phase()
    rf.rotate()
    rf.trim()

    rf.deconv()
    for ff in rf.para.gauss:
        rf.para.rfpath = './F{:.1f}/{}.{}'.format(ff, rf.para.stainfo.network, rf.para.stainfo.station)
        rf.saverf(ff)
    return rf


if __name__ == '__main__':
    test_sub01()


# import folium
from .get_depth import GoodDepth
from seispy.setuplog import setuplog
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication, QWidget, QHBoxLayout,\
                              QVBoxLayout, QSizePolicy, QGroupBox,\
                              QPushButton, QTableWidget, QLineEdit,\
                              QLabel, QAbstractItemView, QTableWidgetItem,\
                              QToolButton, QMessageBox, QFileDialog,\
                              QPlainTextEdit, QHeaderView
from PySide6.QtGui import QIcon, QShortcut, QKeySequence, QGuiApplication
# from PySide6.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np
import sys
from os.path import dirname, join, abspath, isdir, isfile
import os
import logging
import argparse


class LoggerText(logging.Handler):
    def __init__(self, parent):
        super(LoggerText, self).__init__()
        self.widget = parent
        self.widget.setReadOnly(True)

    def emit(self, record):
        msg = self.format(record)
        self.widget.appendPlainText(msg)


class MyMplCanvas(FigureCanvas):
    def __init__(self, parent=None, stack_data_path='', idx=0,
                depmin=30, depmax=60, logger=None, width=7.2,
                height=11, dpi=100, smooth=10):
        plt.rcParams['axes.unicode_minus'] = False 
        self.gooddepth = GoodDepth(stack_data_path, logger, width=width,
                                   height=height, dpi=dpi, smooth=smooth)
        self.gooddepth.get_dep(depmin, depmax)
        self.gooddepth.bin_idx = idx
        self.gooddepth._check_nan_bin()
        self.gooddepth.plot_bin()
        FigureCanvas.__init__(self, self.gooddepth.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QSizePolicy.Expanding,
                                   QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        # FigureCanvas.setCursor(self, Qt.CrossCursor)


class MapUI(QWidget):
    def __init__(self, stack_data_path, depmin=30, depmax=65,
                 width=6, height=11, dpi=100, idx=0, smooth=10):
        super(MapUI, self).__init__()
        self.log = setuplog()
        # self._set_geom_center()
        self.add_log_layout()
        self.map_para = {'width': width, 'height':height, 'dpi':dpi}
        self.mpl = MyMplCanvas(self, stack_data_path=stack_data_path,
                               depmin=depmin, depmax=depmax, idx=idx,
                               logger=self.log, smooth=smooth, **self.map_para)
        self.mpl.mpl_connect('button_press_event', self.on_click)
        self.layout = QHBoxLayout()
        self.layout.addWidget(self.mpl, 7)
        self.add_layout()
        self.setLayout(self.layout)
        self._define_global_shortcuts()
        self.setWindowTitle('Pick Depth')
        self.setWindowIcon(QIcon(join(dirname(dirname(abspath(__file__))), 'data', 'seispy.png')))
    
    def _set_geom_center(self, height=1, width=1):
        screen_resolution = QGuiApplication.primaryScreen().geometry()
        screen_height = screen_resolution.height()
        screen_width = screen_resolution.width()
        frame_height = int(screen_height * height)
        frame_width = int(screen_width * width)

        self.setGeometry(0, 0, frame_width, frame_height)
        self.move(int((screen_width / 2) - (self.frameSize().width() / 2)),
                  int((screen_height / 2) - (self.frameSize().height() / 2)))

    def add_layout(self):
        self.add_save_layout()
        self.add_page_layout()
        self.add_bin_loca_layout()
        # self.add_log_layout()
        ctrlbox = QVBoxLayout()
        # ctrlbox.addStretch(1)
        ctrlbox.addWidget(self.save_box)
        ctrlbox.addWidget(self.page_box)
        ctrlbox.addWidget(self.bin_loca_box)
        ctrlbox.addWidget(self.log_box)
        self.layout.addLayout(ctrlbox, 3)

    def add_save_layout(self):
        self.save_box = QGroupBox('Database saving')
        save_layout = QHBoxLayout()
        self.openbutton = QToolButton()
        self.openbutton.setIcon(QIcon(join(dirname(dirname(abspath(__file__))), 'data', 'saveicon.png')))
        self.openbutton.clicked.connect(self.on_save_file)
        self.save_edit = QLineEdit()
        self.save_edit.setText(join(os.getcwd(), 'good_depths.dat'))
        self.save_edit.setFocusPolicy(Qt.ClickFocus)
        self.savebutton = QPushButton('Save')
        self.savebutton.clicked.connect(self.on_save)
        save_layout.addWidget(self.openbutton, 0)
        save_layout.addWidget(self.save_edit, 9)
        save_layout.addWidget(self.savebutton, 3)
        self.save_box.setLayout(save_layout)

    def add_page_layout(self):
        self.page_box = QGroupBox('Page up/down')
        page_layout = QHBoxLayout()
        # page_layout.addStretch(1)
        self.puButton = QPushButton()
        self.puButton.setText('Back (z)')
        self.puButton.clicked.connect(self.page_up)
        self.pdButton = QPushButton()
        self.pdButton.setText('Next (c)')
        self.pdButton.clicked.connect(self.page_down)
        page_layout.addWidget(self.puButton)
        page_layout.addWidget(self.pdButton)
        self.page_box.setLayout(page_layout)
    
    def add_bin_loca_layout(self):
        self.bin_loca_box = QGroupBox('Bin locations')
        layout = QVBoxLayout()
        show_layout = QHBoxLayout()
        label_bar = QLabel()
        label_bar.setText('Index: ')
        self.idx_edit = QLineEdit()
        self.idx_edit.setText('{}'.format(self.mpl.gooddepth.bin_idx+1))
        self.idx_edit.textChanged.connect(self.on_idx_edit_changed)
        self.idx_edit.setFocusPolicy(Qt.ClickFocus)
        self.loadButton = QPushButton()
        self.loadButton.setText('Load')
        self.loadButton.clicked.connect(self.on_load)
        show_layout.addWidget(label_bar)
        show_layout.addWidget(self.idx_edit)
        show_layout.addWidget(self.loadButton)
        self.set_table()
        layout.addLayout(show_layout)
        layout.addWidget(self.tableWidget)
        self.bin_loca_box.setLayout(layout)

    def add_log_layout(self):
        self.log_box = QGroupBox('Logs')
        layout = QHBoxLayout()
        self.log_text = QPlainTextEdit()
        loghandler = LoggerText(self.log_text)
        self.log.PickDepthlog.addHandler(loghandler)
        # self.log_text.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        layout.addWidget(self.log_text)
        self.log_box.setLayout(layout)

    def set_table(self):
        bin_num = self.mpl.gooddepth.ccp_data.bin_loca.shape[0]
        self.tableWidget = QTableWidget(bin_num, 3)
        self.tableWidget.setHorizontalHeaderLabels(['Latitude', 'Longitude', 'Depth'])
        header = self.tableWidget.horizontalHeader()  
        header.setSectionResizeMode(0, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(1, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(2, QHeaderView.ResizeToContents)
        self.tableWidget.selectRow(self.mpl.gooddepth.bin_idx)
        self.tableWidget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        for i in range(bin_num):
            itemlat = QTableWidgetItem('{:.3f}'.format(self.mpl.gooddepth.ccp_data.bin_loca[i, 0]))
            itemlon = QTableWidgetItem('{:.3f}'.format(self.mpl.gooddepth.ccp_data.bin_loca[i, 1]))
            itemdep = QTableWidgetItem('{:.2f}'.format(self.mpl.gooddepth.good_depth.iloc[i]['depth']))
            self.tableWidget.setItem(i, 0, itemlat)
            self.tableWidget.setItem(i, 1, itemlon)
            self.tableWidget.setItem(i, 2, itemdep)
        self.tableWidget.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableWidget.setSelectionMode(QAbstractItemView.SingleSelection)
        self.tableWidget.cellPressed.connect(self.on_select_row)

    def page_up(self):
        self.mpl.gooddepth.page_up()
        self.mpl.draw_idle()
        self.idx_edit.setText('{}'.format(self.mpl.gooddepth.bin_idx+1))
        self.tableWidget.selectRow(self.mpl.gooddepth.bin_idx)

    
    def page_down(self):
        self.mpl.gooddepth.page_down()
        self.mpl.draw_idle()
        self.idx_edit.setText('{}'.format(self.mpl.gooddepth.bin_idx+1))
        self.tableWidget.selectRow(self.mpl.gooddepth.bin_idx)
    
    def on_select_row(self, event):
        idx = self.tableWidget.currentIndex().row()
        self.mpl.gooddepth.bin_idx = idx
        self.idx_edit.setText('{}'.format(idx+1))

    def on_load(self):
        self.mpl.gooddepth.plot_bin()
        self.mpl.draw()
        self.tableWidget.selectRow(self.mpl.gooddepth.bin_idx)

    def on_save_file(self):
        fedit = self.save_edit.text()
        fname = QFileDialog.getSaveFileName(self,
                'Save as',
                dirname(fedit),
                'All Files (*);; ASCII File (*.dat)')
        self.save_edit.setText(fname[0])

    def on_save(self):
        fname = self.save_edit.text()
        if isdir(fname):
            msg = QMessageBox.critical(self, 'Error',
                 'Cannot create file. {} is a directory.'.format(fname),
                 QMessageBox.Ok)
            return
        if isfile(fname):
            msg = QMessageBox(self)
            msg.setWindowTitle('Warning')
            msg.setText('{} already exists. Do you want to overwrite it.'.format(fname))
            msg.setStandardButtons(QMessageBox.StandardButton.No|QMessageBox.StandardButton.Yes)
            msg.setIcon(QMessageBox.Icon.Warning)
            button = msg.exec()
            if button == QMessageBox.StandardButton.No:
                return
        try:
            self.mpl.gooddepth.write(self.save_edit.text())
            msg = QMessageBox.information(self, 'Information',
                'Successfully save database to {}'.format(fname),
                QMessageBox.Ok)
        except Exception as e:
            msg = QMessageBox.critical(self, 'Error',
                'Error: {}.'.format(e),
                QMessageBox.Ok)
            return

    def on_idx_edit_changed(self):
        text = self.idx_edit.text()
        try:
            self.mpl.gooddepth.bin_idx = int(text)-1
        except:
            self.idx_edit.setText('')
    
    def on_click(self, event):
        bin_idx = self.mpl.gooddepth.bin_idx
        self.mpl.gooddepth.on_click(event)
        itemdep = QTableWidgetItem('{:.2f}'.format(self.mpl.gooddepth.good_depth.iloc[bin_idx]['depth']))
        self.tableWidget.setItem(bin_idx, 2, itemdep)
        self.mpl.draw()
    
    def _define_global_shortcuts(self):
        self.key_c = QShortcut(QKeySequence('c'), self)
        self.key_c.activated.connect(self.page_down)
        self.key_z = QShortcut(QKeySequence('z'), self)
        self.key_z.activated.connect(self.page_up)
        self.key_enter = QShortcut(QKeySequence('Enter'), self)
        self.key_enter.activated.connect(self.on_load)
        self.key_save = QShortcut(QKeySequence.StandardKey.Save, self)
        self.key_save.activated.connect(self.on_save)

           
def main():
    parser = argparse.ArgumentParser(description="User interface for picking PRFs")
    parser.add_argument('stack_data_path', type=str, help='Path to CCP stacked data')
    parser.add_argument('-d', help='Depth range contain target interface', metavar='dep_min/dep_max')
    parser.add_argument('-i', help='Specify starting index of bins', type=int, default=1, metavar='index')
    parser.add_argument('-s', help='Smoothing scale in km', type=float, default=5, metavar='smooth')
    args = parser.parse_args()
    try:
        dep_range = [float(v) for v in args.d.split('/')]
    except:
        raise ValueError('Error format of depth range')
    if args.i < 1:
        raise ValueError('The index should be greater than 0')
    app = QApplication(sys.argv)    
    mapui = MapUI(args.stack_data_path, depmin=dep_range[0], depmax=dep_range[1], idx=args.i-1, smooth=args.s)
    mapui.show()
    sys.exit(app.exec())

if __name__ == '__main__':
    main()


import numpy as np
from scipy.interpolate import griddata, interp1d
from seispy.rfcorrect import DepModel
from seispy.setuplog import setuplog
import argparse


class ModCreator():
    def __init__(self, log=None):
        if log is None:
            self.logger = setuplog()
        else:
            self.logger = log
        self.lats = np.array([])
        self.lons = np.array([])
        self.deps = np.array([])
        self.vps = np.array([])
        self.vss = np.array([])
        self.ddvsddvp = np.array([[150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700],
                                 [1.85, 1.75, 1.75, 1.75, 1.79, 1.75, 2, 1.79, 1.61, 1.67, 1.67, 2.13]])

    def init_grid(self, latmin, latmax, lonmin, lonmax, depmin=0, depmax=850, horival=0.5, depval=50):
        self.logger.ModCreatorlog.info('Setting up grids.')
        self.lat_inter = np.arange(latmin, latmax, horival)
        self.lon_inter = np.arange(lonmin, lonmax, horival)
        self.dep_inter = np.arange(depmin, depmax, depval)
        self.dep_grid, self.lat_grid, self.lon_grid = np.meshgrid(self.dep_inter, self.lat_inter, self.lon_inter, indexing='ij')
        self.depmod = DepModel(self.dep_inter)

    def gridvel(self, **kwargs):
        self.logger.ModCreatorlog.info('Interpolating unstructured velocity to grid')
        points = np.array([self.deps, self.lats, self.lons]).T
        self.vp_grid = griddata(points, self.vps, (self.dep_grid, self.lat_grid, self.lon_grid), **kwargs)
        self.fill_1d('vp')
        if self.vss.size == 0:
            self.calc_vs()
        else:
            self.vs_grid = griddata(points, self.vss, (self.dep_grid, self.lat_grid, self.lon_grid), **kwargs)
            self.fill_1d('vs')

    def calc_vs(self):
        inter_ddvsddvp = interp1d(self.ddvsddvp[0], self.ddvsddvp[1], fill_value="extrapolate")(self.dep_inter)
        mesh_ddvsddvp, _, _ = np.meshgrid(inter_ddvsddvp, self.lat_inter, self.lon_inter, indexing='ij')
        vp_grid1d, _, _ = np.meshgrid(self.depmod.vp, self.lat_inter, self.lon_inter, indexing='ij')
        vs_grid1d, _, _ = np.meshgrid(self.depmod.vs, self.lat_inter, self.lon_inter, indexing='ij')
        self.dvp_grid = ((self.vp_grid - vp_grid1d) / vp_grid1d) * 100
        self.dvs_grid = self.dvp_grid * mesh_ddvsddvp
        self.vs_grid = vs_grid1d * (self.dvs_grid*0.01+1)
    
    def fill_1d(self, type='vp'):
        for i, _ in enumerate(self.dep_inter):
            self.__dict__[type+'_grid'][i][np.where(np.isnan(self.__dict__[type+'_grid'][i, :, :]))] = self.depmod.__dict__[type][i]

    def savenpz(self, filename):
        np.savez(filename, dep=self.dep_inter, lat=self.lat_inter, lon=self.lon_inter, vp=self.vp_grid, vs=self.vs_grid)

    @classmethod
    def read_txt(cls, datfile, usecols=(0, 1, 2, 3, 4), comments='#', delimiter=None):
        mc = cls()
        if len(usecols) == 5:
            mc.lats, mc.lons, mc.deps, mc.vps, mc.vss = np.loadtxt(datfile, unpack=True, usecols=usecols, comments=comments, delimiter=delimiter)
        elif len(usecols) == 4:
            mc.lats, mc.lons, mc.deps, mc.vps = np.loadtxt(datfile, unpack=True, usecols=usecols, comments=comments, delimiter=delimiter)
        else:
            raise ValueError('4 or 5 cols is valid in usecols')
        return mc


def veltxt2mod():
    parser = argparse.ArgumentParser(description="Convert velocity model with text format to npz format for 3-D moveout correction")
    parser.add_argument('velfile', help='velocity file in text format')
    parser.add_argument('-r', help='Range of the grid region with interval in degree',
                        type=str, required=True, metavar='lonmin/lonmax/latmin/latmax/val')
    parser.add_argument('-c', help='Which columns to read, with 0 being the first, order by lat/lon/dep/vp/vs. If 4 columns are accepted,'
                        'the fourth column represents Vp, and the Vs will be calculated as a relationship between'
                        'the Vp/Vs and the depth in Cammarano et al., (2003), defaults to 0/1/2/3/4',
                        type=str, default='0/1/2/3/4', metavar='ncol_lat/ncol_lon/ncol_dep/ncol_vp[/ncol_vs]')
    parser.add_argument('-d', help='Range of the grid region with interval in km, defaults to 0/850/50', 
                        type=str, default='0/850/50', metavar='depmin/depmax/val')
    parser.add_argument('-m', help='Method of interpolation. One of \'linear\', \'nearest\', \'cubic\', defaults to \'linear\'',
                        type=str, default='linear', metavar='linear|nearest|cubic')
    parser.add_argument('-o', help='output filename with \'npz\' format, defaults to ./mod3d.npz', default='./mod3d.npz')
    args = parser.parse_args()
    try:
        cols = [int(value) for value in args.c.split('/')]
    except:
        raise ValueError('Error in parsing -c option')
    try:
        gridrange = [float(value) for value in args.r.split('/')]
    except:
        raise ValueError('Error in parsing -r option')
    try:
        deprange = [float(value) for value in args.d.split('/')]
    except:
        raise ValueError('Error in parsing -d option')
    if args.m not in ['linear', 'nearest', 'cubic']:
        raise ValueError('The method of interpolation must be in \'linear\', \'nearest\', \'cubic\'')

    mc = ModCreator.read_txt(args.velfile, usecols=cols)
    mc.init_grid(gridrange[2], gridrange[3], gridrange[0], gridrange[1], depmin=deprange[0],
                 depmax=deprange[1], horival=gridrange[4], depval=deprange[2])
    mc.gridvel(method=args.m)
    mc.savenpz(args.o)


if __name__ == '__main__':
    veltxt2mod()




import numpy as np
import obspy
from obspy.io.sac import SACTrace
from obspy.signal.rotate import rotate2zne, rotate_zne_lqt
from scipy.signal import resample
from os.path import join
from seispy.decon import RFTrace
from seispy.geo import snr, srad2skm, rotateSeisENtoTR, \
                       rssq, extrema
from obspy.signal.trigger import recursive_sta_lta
import glob


class NotEnoughComponent(Exception):
    def __init__(self, matchkey):
        self.matchkey = matchkey
    def __str__(self):
        return '{}'.format(self.matchkey)

class TooMoreComponents(Exception):
    def __init__(self, matchkey):
        self.matchkey = matchkey
    def __str__(self):
        return '{}'.format(self.matchkey)

def rotateZNE(st):
    try:
        zne = rotate2zne(
            st[0], st[0].stats.sac.cmpaz, st[0].stats.sac.cmpinc,
            st[1], st[1].stats.sac.cmpaz, st[1].stats.sac.cmpinc,
            st[2], st[2].stats.sac.cmpaz, st[2].stats.sac.cmpinc)
    except Exception as e:
        raise ValueError('No specified cmpaz or cmpinc. {}'.format(e))
    for tr, new_data, component in zip(st, zne, "ZNE"):
        tr.data = new_data
        tr.stats.channel = tr.stats.channel[:-1] + component


class EQ(object):
    def __init__(self, pathname, datestr, suffix='SAC'):
        """Class for processing event data with 3 components, which read SAC files of ``pathname*datastr*suffix`` 

        :param pathname: Directory to SAC files
        :type pathname: string
        :param datestr: date part in filename, e.g., ``2021.122.12.23.40``
        :type datestr: string
        :param suffix: suffix for SAC files, defaults to 'SAC'
        :type suffix: str, optional
        """
        self.datestr = datestr
        self.filestr = join(pathname, '*' + datestr + '*' + suffix)
        if glob.glob(self.filestr):  
            self.st = obspy.read(self.filestr)
            self._check_comp()
            self.st.sort()
            self.set_comp()
        else:
            self.st = obspy.Stream()
        self.rf = obspy.Stream()
        self.timeoffset = 0
        self.rms = np.array([0])
        self.it = 0
        self.trigger_shift = 0
        self.inc_correction = 0
    
    def _check_comp(self):
        if len(self.st) < 3:
            channel = ' '.join([tr.stats.channel for tr in self.st])
            raise NotEnoughComponent('Sismogram must be in 3 components, but there are only channel {} of {}'.format(channel, self.datestr))
        elif len(self.st) > 3:
            raise TooMoreComponents('{} has more than 3 components, please select to delete redundant seismic components'.format(self.datestr))
        else:
            pass

    def readstream(self):
        self.rf = obspy.Stream()
        self.st = obspy.read(self.filestr)
        self._check_comp()
        self.st.sort()
        self.set_comp()

    def cleanstream(self):
        self.st = None
        self.rf = None

    @classmethod
    def from_stream(cls, stream):
        eq = cls('', '')
        eq.st = stream
        eq.datestr = eq.st[0].stats.starttime.strftime('%Y.%j.%H.%M.%S')
        eq._check_comp()
        eq.st.sort()
        eq.set_comp()
        return eq

    def set_comp(self):
        if self.st.select(channel='*[E2]'):
            self.comp = 'enz'
        elif self.st.select(channel='*R'):
            self.comp = 'rtz'
        elif self.st.select(channel='*Q'):
            self.comp = 'lqt'
        else:
            raise ValueError('No such component in R, E or Q')

    def channel_correct(self, switchEN=False, reverseE=False, reverseN=False):
        """_summary_

        :param switchEN: _description_, defaults to False
        :type switchEN: bool, optional
        :param reverseE: _description_, defaults to False
        :type reverseE: bool, optional
        :param reverseN: _description_, defaults to False
        :type reverseN: bool, optional
        """
        if reverseE:
            self.st.select(channel='*[E2]')[0].data *= -1
        if reverseN:
            self.st.select(channel='*[N1]')[0].data *= -1
        if switchEN:
            chE = self.st.select(channel='*[E2]')[0].stats.channel
            chN = self.st.select(channel='*[N1]')[0].stats.channel
            self.st.select(channel='*[E2]')[0].stats.channel = chN
            self.st.select(channel='*[N1]')[0].stats.channel = chE

    def __str__(self):
        return('Event data class {0}'.format(self.datestr))

    def write(self, path, evt_datetime):
        for tr in self.st:
            sac = SACTrace.from_obspy_trace(tr)
            sac.b = 0
            sac.o = evt_datetime - tr.stats.starttime
            fname = join(path, '{}.{}.{}.{}.SAC'.format(
                tr.stats.network, tr.stats.station,
                tr.stats.starttime.strftime('%Y.%j.%H%M%S'),
                tr.stats.channel
            ))
            sac.write(fname)

    def detrend(self):
        self.st.detrend(type='linear')
        self.st.detrend(type='constant')
        self.fix_channel_name()

    def filter(self, freqmin=0.05, freqmax=1, order=4):
        self.st.filter('bandpass', freqmin=freqmin, freqmax=freqmax, corners=order, zerophase=True)

    def get_arrival(self, model, evdp, dis, phase='P'):
        arrivals = model.get_travel_times(evdp, dis, phase_list=[phase])
        if not arrivals:
            raise ValueError('The phase of {} is not exists'.format(phase))
        if len(arrivals) > 1:
            raise ValueError('More than one phase were calculated with distance of {} and focal depth of {}'.format(dis, evdp))
        else:
            # self.arrival = arrivals[0]
            self.arr_time = arrivals[0].time
            self.rayp = arrivals[0].ray_param
            self.inc = arrivals[0].incident_angle
            self.phase = phase

    def search_inc(self, bazi):
        inc_range = np.arange(0.1, 90, 0.1)
        s_range = self.trim(20, 20, isreturn=True)
        power = np.zeros(inc_range.shape[0])
        for i in range(len(inc_range)):
            l_comp, _, _ = rotate_zne_lqt(s_range[2].data, s_range[1].data, s_range[0].data, bazi, inc_range[i])
            power[i] = np.mean(l_comp ** 2)

        real_inc_idx = np.argmin(power)
        real_inc = inc_range[real_inc_idx]
        self.inc_correction = real_inc - self.inc
        self.inc = real_inc

    def search_baz(self, bazi, time_b=10, time_e=20, offset=90):
        p_arr = self.arr_correct(write_to_sac=False)
        this_st = self.st.copy()
        this_st.filter('bandpass', freqmin=0.03, freqmax=0.5)
        this_st.trim(this_st[0].stats.starttime+p_arr-time_b, this_st[0].stats.starttime+p_arr+time_e)
        bazs = np.arange(-offset, offset) + bazi
        ampt = np.zeros_like(bazs)
        for i, b in enumerate(bazs):
            t, _ = rotateSeisENtoTR(this_st[0].data, this_st[1].data, b)
            ampt[i] = rssq(t)
        ampt = ampt / np.max(ampt)
        idx = extrema(ampt, opt='min')
        if len(idx) == 0:
            corr_baz = np.nan
        elif len(idx) > 1:
            corr_baz = None
        else:
            corr_baz = bazs[idx[0]] - bazi
        return corr_baz, ampt

    def fix_channel_name(self):
        if self.st.select(channel='??1') and self.st.select(channel='??Z') and hasattr(self.st.select(channel='*1')[0].stats.sac, 'cmpaz'):
            if self.st.select(channel='*1')[0].stats.sac.cmpaz == 0:
                self.st.select(channel='*1')[0].stats.channel = self.st.select(channel='*1')[0].stats.channel[:-1] + 'N'
                self.st.select(channel='*2')[0].stats.channel = self.st.select(channel='*2')[0].stats.channel[:-1] + 'E'
            elif self.st.select(channel='*1')[0].stats.sac.cmpaz != 0:
                rotateZNE(self.st)
            self.st.sort(['channel'])
        elif self.st.select(channel='*1'):
            self.st.select(channel='*1')[0].stats.channel = 'Z'
            self.st.select(channel='*2')[0].stats.channel = 'N'
            self.st.select(channel='*3')[0].stats.channel = 'E'
            self.st.sort(['channel'])
        else:
            pass

    def rotate(self, baz, inc=None, method='NE->RT', search_inc=False, baz_shift=0):
        bazi = np.mod(baz + baz_shift, 360)
        if inc is None:
            if self.phase[-1] == 'S' and search_inc:
                inc = self.search_inc(bazi)
        else:
            self.inc = inc

        if method == 'NE->RT':
            self.comp = 'rtz'
            self.st.rotate('NE->RT', back_azimuth=bazi)
        elif method == 'RT->NE':
            self.st.rotate('RT->NE', back_azimuth=bazi)
        elif method == 'ZNE->LQT':
            self.comp = 'lqt'
            self.st.rotate('ZNE->LQT', back_azimuth=bazi, inclination=self.inc)
        elif method == 'LQT->ZNE':
            self.st.rotate('LQT->ZNE', back_azimuth=bazi, inclination=self.inc)
        else:
            pass

    def snr(self, length=50):
        st_noise = self.trim(length, 0, isreturn=True)
        st_signal = self.trim(0, length, isreturn=True)
        try:
            snr_E = snr(st_signal[0].data, st_noise[0].data)
        except IndexError:
            snr_E = 0
        try:
            snr_N = snr(st_signal[1].data, st_noise[1].data)
        except IndexError:
            snr_N = 0
        try:
            snr_Z = snr(st_signal[2].data, st_noise[2].data)
        except IndexError:
            snr_Z = 0
        return snr_E, snr_N, snr_Z
    
    def get_time_offset(self, event_time=None):
        if event_time is not None and not isinstance(event_time, obspy.core.utcdatetime.UTCDateTime):
            raise TypeError('Event time should be UTCDateTime type in obspy')
        elif event_time is None:
            self.timeoffset = self.st[2].stats.sac.b - self.st[2].stats.sac.o
        else:
            self.timeoffset = self.st[2].stats.starttime - event_time

    def arr_correct(self, write_to_sac=True):
        """
        offset = sac.b - real o
        """
        correct_time = self.arr_time - self.timeoffset
        if write_to_sac:
            for tr in self.st:
                tr.stats.sac.t0 = correct_time
                tr.stats.sac.kt0 = self.phase

        return correct_time

    def _get_time(self, time_before, time_after):
        arr = self.arr_correct(write_to_sac=False)
        t1 = self.st[2].stats.starttime + (arr + self.trigger_shift - time_before)
        t2 = self.st[2].stats.starttime + (arr + self.trigger_shift + time_after)
        return t1, t2

    def phase_trigger(self, time_before, time_after, stl=5, ltl=10):
        t1, t2 = self._get_time(time_before, time_after)
        self.st_pick = self.st.copy().trim(t1, t2)
        if len(self.st_pick) == 0:
            return
        if self.phase[-1] == 'P':
            tr = self.st_pick.select(channel='*Z')[0]
        else:
            tr = self.st_pick.select(channel='*T')[0]
        df = tr.stats.sampling_rate
        cft = recursive_sta_lta(tr.data, int(stl*df), int(ltl*df))
        n_trigger = np.argmax(np.diff(cft)[int(ltl*df):])+int(ltl*df)
        self.t_trigger = t1 + n_trigger/df
        self.trigger_shift = n_trigger/df - time_before

    def trim(self, time_before, time_after, isreturn=False):
        """
        offset = sac.b - real o
        """
        t1, t2 = self._get_time(time_before, time_after)
        if isreturn:
            return self.st.copy().trim(t1, t2)
        else:
            self.st.trim(t1, t2)

    def deconvolute(self, shift, time_after, f0=2.0, method='iter', only_r=False,
                    itmax=400, minderr=0.001, wlevel=0.05, target_dt=None):
        """Deconvolution

        Parameters
        ----------
        shift : float
            Time shift before P arrival
        time_after : float
            Time length after P arrival
        f0 : float or list, optional
            Gaussian factors, by default 2.0
        method : str, optional
            method for deconvolution in ``iter`` or ``water``, by default ``iter``
        only_r : bool, optional
            Whether only calculate RF in prime component, by default False
        itmax : int, optional
            Maximum iterative number, valid for method of ``iter``, by default 400
        minderr : float, optional
            Minium residual error, valid for method of ``iter``, by default 0.001
        wlevel : float, optional
            Water level, valid for method of ``water``, by default 0.05
        target_dt : None or float, optional
            Time delta for resampling, by default None
        """
        self.method = method
        if isinstance(f0, (int, float)):
            f0 = [f0]
        for ff in f0:
            if method == 'iter':
                kwargs = {'method': method,
                        'f0': ff,
                        'tshift': shift,
                        'itmax': itmax,
                        'minderr': minderr}
            elif method == 'water':
                kwargs = {'method': method,
                        'f0': ff,
                        'tshift': shift,
                        'wlevel': wlevel}
            else:
                raise ValueError('method must be in \'iter\' or \'water\'')

            if self.phase[-1] == 'P':
                self.decon_p(**kwargs)
                if not only_r:
                    self.decon_p(tcomp=True, **kwargs)
            else:
                # TODO: if 'Q' not in self.rf[1].stats.channel or 'L' not in self.rf[2].stats.channel:
                #     raise ValueError('Please rotate component to \'LQT\'')
                self.decon_s(**kwargs)
            if target_dt is not None:
                for tr in self.rf:
                    if tr.stats.delta != target_dt:
                        tr.data = resample(tr.data, int((shift + time_after)/target_dt+1))
                        tr.stats.delta = target_dt
    
    def decon_p(self, tshift, tcomp=False, **kwargs):
        if self.comp == 'lqt':
            win = self.st.select(channel='*L')[0]
            if tcomp:
                uin = self.st.select(channel='*T')[0]
            else:
                uin = self.st.select(channel='*Q')[0]
                uin.data *= -1
        else:
            win = self.st.select(channel='*Z')[0]
            if tcomp:
                uin = self.st.select(channel='*T')[0]
            else:
                uin = self.st.select(channel='*R')[0]
        uout = RFTrace.deconvolute(uin, win, phase='P', tshift=tshift, **kwargs)
        self.rf.append(uout)

    def decon_s(self, tshift, **kwargs):
        if self.comp == 'lqt':
            win = self.st.select(channel='*Q')[0]
            uin = self.st.select(channel='*L')[0]
        else:
            win = self.st.select(channel='*R')[0]
            uin = self.st.select(channel='*Z')[0]
            win.data *= -1
        # win.data[0:int((tshift-4)/win.stats.delta)] = 0
        uout = RFTrace.deconvolute(uin, win, phase='S', tshift=tshift, **kwargs)
        uout.data = np.flip(uout.data)
        self.rf.append(uout)

    def saverf(self, path, evtstr=None, shift=0, evla=-12345., evlo=-12345., evdp=-12345., mag=-12345.,
               gauss=0, baz=-12345., gcarc=-12345., only_r=False, **kwargs):
        if self.phase[-1] == 'P':
            if self.comp == 'lqt':
                svcomp = 'Q'
            else:
                svcomp = 'R'
            if only_r:
                loop_lst = [svcomp]
            else:
                loop_lst = [svcomp, 'T']
            rayp = srad2skm(self.rayp)
        elif self.phase[-1] == 'S':
            if self.comp == 'lqt':
                loop_lst = ['L']
            else:
                loop_lst = ['Z']
            rayp = srad2skm(self.rayp)
        else:
            pass
        if evtstr is None:
            filename = join(path, self.datestr)
        else:
            filename = join(path, evtstr)
        for comp in loop_lst:
            trrfs = self.rf.select(channel='*'+comp)
            try:
                trrf = [tr for tr in trrfs if tr.stats.f0 == gauss][0]
            except:
                ValueError('No such gauss factor of {} in calculated RFs'.format(gauss))
            header = {'evla': evla, 'evlo': evlo, 'evdp': evdp, 'mag': mag, 'baz': baz,
                      'gcarc': gcarc, 'user0': rayp, 'kuser0': 'Ray Para', 'user1': gauss, 'kuser1': 'G factor'}
            for key in kwargs:
                header[key] = kwargs[key]
            for key, value in header.items():
                trrf.stats['sac'][key] = value
            tr = SACTrace.from_obspy_trace(trrf)
            tr.b = -shift
            tr.a = 0
            tr.ka = self.phase
            tr.write(filename + '_{0}_{1}.sac'.format(self.phase, tr.kcmpnm[-1]))

    def s_condition(self, trrf, shift):
        nt0 = int(np.floor((shift)/trrf.stats.delta))
        nt25 = int(np.floor((shift+25)/trrf.stats.delta))
        if rssq(trrf.data[nt0:nt25]) > rssq(trrf.data[nt25:]):
            return True
        else:
            return False

    def judge_rf(self, gauss, shift, npts, criterion='crust', rmsgate=None):
        if self.phase[-1] == 'P' and self.comp == 'rtz':
            trrfs = self.rf.select(channel='*R')
        elif self.phase[-1] == 'P' and self.comp == 'lqt':
            trrfs = self.rf.select(channel='*Q')
        elif self.phase[-1] == 'S' and self.comp == 'lqt':
            trrfs = self.rf.select(channel='*L')
        elif self.phase[-1] == 'S' and self.comp == 'rtz':
            trrfs = self.rf.select(channel='*Z')
        for tr in trrfs:      
            if tr.stats.npts != npts:
                return False
        try:
            trrf = [tr for tr in trrfs if tr.stats.f0 == gauss][0]
        except:
            ValueError('No such gauss factor of {} in calculated RFs'.format(gauss))
        
        # All points are NaN
        if np.isnan(trrf.data).all():
            return False
        
        if np.isinf(trrf.data).any():
            return False
        
        # Final RMS
        if rmsgate is not None:
            if self.method == 'iter':
                rms = trrf.stats.rms[-1]
            else:
                rms = trrf.stats.rms
            rmspass = rms < rmsgate
        else:
            rmspass = True
        
        # R energy
        nt1 = int(np.floor((5+shift)/trrf.stats.delta))
        reng = np.sum(np.sqrt(trrf.data[nt1:] ** 2))
        if reng < 0.1:
            rengpass = False
        else:
            rengpass = True

        # Max amplitude
        if criterion == 'crust':
            time_P1 = int(np.floor((-2+shift)/trrf.stats.delta))
            time_P2 = int(np.floor((4+shift)/trrf.stats.delta))
            max_P = np.max(trrf.data[time_P1:time_P2])
            if max_P == np.max(np.abs(trrf.data)) and max_P < 1 and rmspass and rengpass:
                return True and rengpass
            else:
                return False
        elif criterion == 'mtz':
            max_deep = np.max(np.abs(trrf.data[int((30 + shift) / trrf.stats.delta):]))
            time_P1 = int(np.floor((-5 + shift) / trrf.stats.delta))
            time_P2 = int(np.floor((5 + shift) / trrf.stats.delta))
            max_P = np.max(trrf.data[time_P1:time_P2])
            if max_deep < max_P * 0.4 and rmspass and rengpass and\
                  max_P == np.max(np.abs(trrf.data)) and max_P < 1:
                return True and rengpass
            else:
                return False
        elif criterion == "lab":
            return self.s_condition(trrf, shift) and rengpass
        elif criterion is None:
            return rmspass and rengpass
        else:
            pass


if __name__ == '__main__':
    pass




from seispy.utils import DepModel
from seispy.seisfwd import SynSeis
from seispy.rfcorrect import RFStation
import pytest
import numpy as np


def test_sub01():
    depth = np.array([0, 20.1, 35.1, 100])
    model = DepModel(depth)
    rayp = np.arange(0.04, 0.09, 0.01)
    ss = SynSeis(model, rayp, 0.1, 2400)
    ss.run_fwd()
    rfs = ss.run_deconvolution()
    rfsta = RFStation.read_stream(rfs, rayp, 0)
    rfsta.moveoutcorrect(ref_rayp=0.06, dep_range=np.arange(100))


if __name__ == '__main__':
    test_sub01()


from obspy import UTCDateTime
import argparse
from seispy.io import Query
import sys
import pandas as pd

def download_catalog(fname, server='IRIS', format='seispy', **kwargs):
    query = Query(server=server)
    query.get_events(**kwargs)
    write_catalog(query, fname, format)


def write_catalog(query, fname, format):
    if format == 'seispy':
        with open(fname, 'w') as f:
            for i, row in query.events.iterrows():
                f.write('{} {:.3f} {:.3f} {:.1f} {:.1f}\n'.format(
                    row['date'].strftime('%Y %m %d %j %H %M %S'),
                    row['evla'], row['evlo'], row['evdp'], row['mag']
                ))
    else:
        query.events_raw.write(fname, format=format)


def read_catalog_file(fname):
    col = ['date', 'evla', 'evlo', 'evdp', 'mag']
    colcata = ['year', 'month', 'day', 'hour', 'minute', 'second']
    data_cata = pd.read_table(fname, header=None, sep=' |\t', engine='python')
    date_cols = data_cata.loc[:, [0,1,2,4,5,6]]
    date_cols.columns = colcata
    dates = pd.to_datetime(date_cols)
    eq_lst = pd.concat([dates, data_cata.loc[:, 7:]], axis=1)
    eq_lst.columns = col
    return eq_lst


def main():
    parser = argparse.ArgumentParser(description="Download Catalog to local file")
    parser.add_argument('fname', help='File name of output catalog')
    parser.add_argument('-f', help='The file format to use (e.g. "QUAKEML"). '
                        'See https://docs.obspy.org/master/packages/autogen/obspy.core.event.Catalog.write.html#supported-formats'
                        ' for supported formats. In addition, the \'seispy\' is supported for writing to a text file',
                         metavar='format', default='seispy')
    parser.add_argument('-b', help='Start time following obspy.UTCDateTime format', default=None, metavar='starttime')
    parser.add_argument('-e', help='End time following obspy.UTCDateTime format', default=None, metavar='endtime')
    parser.add_argument('-d', help='Radial geographic constraints with <lat>/<lon>/<minradius>/<maxradius>',
                        metavar='<lat>/<lon>/<minradius>/<maxradius>', default=None)
    parser.add_argument('-m', help='Magnitude <minmagnitude>[/<maxmagnitude>]',
                        metavar='<minmagnitude>[/<maxmagnitude>]', default=None)
    parser.add_argument('-p', help='Focal depth <mindepth>[/<maxdepth>]', metavar='<mindepth>[/<maxdepth>]', default=None)
    parser.add_argument('-s', help='Service provider, defaults to IRIS', metavar='server', default='IRIS')
    parser.add_argument('-c', help='Catalog name, defaults to GCMT', metavar='catalog', default='GCMT')
    arg = parser.parse_args()
    args = {}
    if arg.c is not None:
        args['catalog'] = arg.c
    if arg.p is not None:
        try:
            values = [float(value) for value in arg.p.split('/')]
        except:
            raise ValueError('Error format with focal depth')
        if len(values) == 1:
            args['mindepth'] = values[0]
        elif len(values) == 2:
            args['mindepth'] = values[0]
            args['maxdepth'] = values[1]
        else:
            raise ValueError('Error format with focal depth')
    if arg.m is not None:
        try:
            values = [float(value) for value in arg.m.split('/')]
        except:
            raise ValueError('Error format with magnitude')
        if len(values) == 1:
            args['minmagnitude'] = values[0]
        elif len(values) == 2:
            args['minmagnitude'] = values[0]
            args['maxmagnitude'] = values[1]
        else:
            raise ValueError('Error format with focal depth')
    if arg.d is not None:
        try:
            values = [float(value) for value in arg.d.split('/')]
        except:
            raise ValueError('Error format with Radial geographic constraints')
        if len(values) == 4:
            args['latitude'] = values[0]
            args['longitude'] = values[1]
            args['minradius'] = values[2]
            args['maxradius'] = values[3]
        else:
            raise ValueError('Error format with radial geographic constraints')
    if arg.b is not None:
        try:
            args['starttime'] = UTCDateTime(arg.b)
        except:
            raise ValueError('-b: Error format with time string')
    if arg.e is not None:
        try:
            args['endtime'] = UTCDateTime(arg.e)
        except:
            raise ValueError('-e: Error format with time string')
    if args == {}:
        parser.print_usage()
        sys.exit(1)
    download_catalog(arg.fname, server=arg.s, format=arg.f, **args)

